import { ICommandDefinition, ICommandProfileTypeConfiguration } from "../../cmd";
import { IImperativeConfig } from "./doc/IImperativeConfig";
/**
 * This class is used to contains all configuration being set by Imperative.
 * It is a singleton ans should be accessed via ImperativeConfig.instance.
 */
export declare class ImperativeConfig {
    /**
     * This is the variable that store the specific instance of Imperative Config.
     * Defined as static so that it can be accessed from anywhere.
     *
     * @private
     * @type {ImperativeConfig}
     */
    private static mInstance;
    /**
     * This variable is used to contains an instance of the Logger object.
     *
     * @private
     * @type {Logger}
     */
    private console;
    /**
     * This paramereter is used as the container of all loaded configuration for
     * Imperative.
     *
     * @private
     * @type {IImperativeConfig}
     */
    private mLoadedConfig;
    /**
     * This paramereter is used to contain the caller location of imperative configuration file.
     * Imperative.
     *
     * @private
     * @type {string}
     */
    private mCallerLocation;
    /**
     * This is the package name of the host application. It will only be set once accessed to
     * lessen loads to the host package.json.
     */
    private mHostPackageName;
    /**
     * This is the name of our imperative package. It will only be set once accessed to
     * lessen loads to the imperative package.json.
     *
     * It isn't hardcoded so that the name of our package can change without affecting
     * modules dependent on it.
     */
    private mImperativePackageName;
    /**
     * Gets a single instance of the PluginIssues. On the first call of
     * ImperativeConfig.instance, a new Plugin Issues object is initialized and returned.
     * Every subsequent call will use the one that was first created.
     *
     * @returns {ImperativeConfig} The newly initialized PMF object.
     */
    static readonly instance: ImperativeConfig;
    /**
     * Set the caller location.
     * @param {string} location new location to be updated with
     */
    /**
    * Return file location of imperative configuration file.
    * @returns {string} - location of configuration file
    */
    callerLocation: string;
    /**
     * Set the loaded config data.
     * @param {IImperativeConfig} config to be set.
     */
    /**
    * Retrieve the loaded config (if init has
    * @returns {IImperativeConfig} - the config that has been loaded, if any
    */
    loadedConfig: IImperativeConfig;
    /**
     * Retrieve the host package name from which imperative was called.
     */
    readonly hostPackageName: string;
    /**
     * Retrieve the package name of the imperative application.
     */
    readonly imperativePackageName: string;
    /**
     * Add a new command group by inserting it to the definitions list of the loaded config.
     * @param {ICommandDefinition} cmdDefToAdd - command definition group to to be added.
     */
    addCmdGrpToLoadedConfig(cmdDefToAdd: ICommandDefinition): void;
    /**
     * Add a new set of profiles by inserting them into the profiles of the loaded config.
     * @param {ICommandProfileTypeConfiguration[]} profiles
     *    Array of profiles to be added.
     */
    addProfiles(profiles: ICommandProfileTypeConfiguration[]): void;
    /**
     * Parses the package.json file and searches for the symlink name used under "bin".
     * @returns {string} - return bin symlink name if present, otherwise null
     */
    findPackageBinName(): string;
    /**
     * Return the cli Home path.
     * @return {string} path to cli Home.
     */
    readonly cliHome: string;
    /**
     * Return profile Directory.
     * @return {string} profile directory.
     */
    readonly profileDir: string;
    /**
     * Get imperative's host CLI command tree with all module globs resolved.
     *
     * @return {ICommandDefinition} The resolved command tree
     */
    readonly resolvedCmdTree: ICommandDefinition;
    /**
     * Get imperative's host CLI command tree after final preparation.
     *
     * @param {IImperativeConfig} resolvedCmdTree - The imperative command tree
     *        returned by ImperativeConfig.resolvedCmdTree()
     */
    getPreparedCmdTree(resolvedCmdTree: ICommandDefinition): ICommandDefinition;
    /**
     * Return package.json of the imperative user
     * @returns {any} - package.json file of caller
     */
    readonly callerPackageJson: any;
    /**
     * Require a file from a project using imperative accounting for imperative being contained
     * separately from the current implementers directory.
     * @param {string} file - the file to require from project using imperative
     */
    getCallerFile(file: string): any;
    /**
     * Append any auto generated commands to the root command document depending on configuration.
     * @param {ICommandDefinition} rootCommand - the root command as built so far
     * @returns {ICommandDefinition} - the root command with any auto generated commands appended
     */
    private addAutoGeneratedCommands;
}
