"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
const cmd_1 = require("../../cmd");
const CompleteProfilesGroupBuilder_1 = require("./profiles/builders/CompleteProfilesGroupBuilder");
const DefinitionTreeResolver_1 = require("./DefinitionTreeResolver");
const logger_1 = require("../../logger");
const path_1 = require("path");
const error_1 = require("../../error");
const EnvironmentalVariableSettings_1 = require("./env/EnvironmentalVariableSettings");
/**
 * This class is used to contains all configuration being set by Imperative.
 * It is a singleton ans should be accessed via ImperativeConfig.instance.
 */
class ImperativeConfig {
    constructor() {
        /**
         * This variable is used to contains an instance of the Logger object.
         *
         * @private
         * @type {Logger}
         */
        this.console = logger_1.Logger.getImperativeLogger();
        /**
         * This paramereter is used as the container of all loaded configuration for
         * Imperative.
         *
         * @private
         * @type {IImperativeConfig}
         */
        this.mLoadedConfig = null;
        /**
         * This paramereter is used to contain the caller location of imperative configuration file.
         * Imperative.
         *
         * @private
         * @type {string}
         */
        this.mCallerLocation = null;
    }
    /**
     * Gets a single instance of the PluginIssues. On the first call of
     * ImperativeConfig.instance, a new Plugin Issues object is initialized and returned.
     * Every subsequent call will use the one that was first created.
     *
     * @returns {ImperativeConfig} The newly initialized PMF object.
     */
    static get instance() {
        if (this.mInstance == null) {
            this.mInstance = new ImperativeConfig();
        }
        return this.mInstance;
    }
    /**
     * Set the caller location.
     * @param {string} location new location to be updated with
     */
    set callerLocation(location) {
        this.mCallerLocation = location;
    }
    /**
     * Return file location of imperative configuration file.
     * @returns {string} - location of configuration file
     */
    get callerLocation() {
        return this.mCallerLocation;
    }
    /**
     * Set the loaded config data.
     * @param {IImperativeConfig} config to be set.
     */
    set loadedConfig(config) {
        this.mLoadedConfig = config;
    }
    /**
     * Retrieve the loaded config (if init has
     * @returns {IImperativeConfig} - the config that has been loaded, if any
     */
    get loadedConfig() {
        return this.mLoadedConfig;
    }
    /**
     * Retrieve the host package name from which imperative was called.
     */
    get hostPackageName() {
        if (!this.mHostPackageName) {
            this.mHostPackageName = this.callerPackageJson.name;
        }
        return this.mHostPackageName;
    }
    /**
     * Retrieve the package name of the imperative application.
     */
    get imperativePackageName() {
        if (!this.mImperativePackageName) {
            this.mImperativePackageName = require(path_1.join(__dirname, "../../../package.json")).name;
        }
        return this.mImperativePackageName;
    }
    /**
     * Add a new command group by inserting it to the definitions list of the loaded config.
     * @param {ICommandDefinition} cmdDefToAdd - command definition group to to be added.
     */
    addCmdGrpToLoadedConfig(cmdDefToAdd) {
        if (this.loadedConfig != null) {
            if (this.loadedConfig.definitions == null) {
                this.loadedConfig.definitions = [];
            }
            const defIndex = this.loadedConfig.definitions.indexOf(cmdDefToAdd);
            if (defIndex > -1) {
                this.loadedConfig.definitions.splice(defIndex, 1);
            }
            this.console.debug("Adding definition = '" + cmdDefToAdd.name + "'");
            this.loadedConfig.definitions.push(cmdDefToAdd);
        }
    }
    /**
     * Add a new set of profiles by inserting them into the profiles of the loaded config.
     * @param {ICommandProfileTypeConfiguration[]} profiles
     *    Array of profiles to be added.
     */
    addProfiles(profiles) {
        if (this.loadedConfig) {
            if (!this.loadedConfig.profiles) {
                this.loadedConfig.profiles = [];
            }
            for (const profileToAdd of profiles) {
                /* We expect to not find (ie, undefined result) an exiting profile
                 * with the same type value as the profile that we want to add.
                 */
                const existingProfile = this.loadedConfig.profiles.find((profileToTest) => {
                    return profileToTest.type === profileToAdd.type;
                });
                if (existingProfile) {
                    this.console.error("addProfiles: The profile of type '" + profileToAdd.type +
                        "' already exists. It will not be added.");
                    continue;
                }
                this.console.debug("addProfiles: Adding " + profileToAdd.type + " profile");
                this.loadedConfig.profiles.push(profileToAdd);
            } // end for
        } // end if loadedConfig not null
    }
    /**
     * Parses the package.json file and searches for the symlink name used under "bin".
     * @returns {string} - return bin symlink name if present, otherwise null
     */
    findPackageBinName() {
        const pkg = this.callerPackageJson;
        if (typeof pkg.bin === "string") {
            return pkg.name;
        }
        else if (typeof pkg.bin === "object") {
            return Object.keys(pkg.bin).pop();
        }
        return null;
    }
    /**
     * Return the cli Home path.
     * @return {string} path to cli Home.
     */
    get cliHome() {
        const settings = EnvironmentalVariableSettings_1.EnvironmentalVariableSettings.read(this.loadedConfig.envVariablePrefix || this.loadedConfig.name);
        if (settings.cliHome.value != null) {
            return settings.cliHome.value;
        }
        return this.loadedConfig.defaultHome;
    }
    /**
     * Return profile Directory.
     * @return {string} profile directory.
     */
    get profileDir() {
        return this.loadedConfig.defaultHome + constants_1.Constants.PROFILES_DIR + "/";
    }
    /**
     * Get imperative's host CLI command tree with all module globs resolved.
     *
     * @return {ICommandDefinition} The resolved command tree
     */
    get resolvedCmdTree() {
        const config = this.loadedConfig;
        return DefinitionTreeResolver_1.DefinitionTreeResolver.resolve(config.rootCommandDescription || "", config.productDisplayName, path_1.dirname(this.callerLocation), this.console, config.definitions, config.commandModuleGlobs);
    }
    /**
     * Get imperative's host CLI command tree after final preparation.
     *
     * @param {IImperativeConfig} resolvedCmdTree - The imperative command tree
     *        returned by ImperativeConfig.resolvedCmdTree()
     */
    getPreparedCmdTree(resolvedCmdTree) {
        let preparedCmdTree = this.addAutoGeneratedCommands(resolvedCmdTree);
        preparedCmdTree = cmd_1.CommandPreparer.prepare(preparedCmdTree);
        return preparedCmdTree;
    }
    /**
     * Return package.json of the imperative user
     * @returns {any} - package.json file of caller
     */
    get callerPackageJson() {
        return this.getCallerFile("package.json");
    }
    /**
     * Require a file from a project using imperative accounting for imperative being contained
     * separately from the current implementers directory.
     * @param {string} file - the file to require from project using imperative
     */
    getCallerFile(file) {
        // try to locate the file using find-up first
        let findupErr;
        try {
            const filePath = require("find-up").sync(file, { cwd: ImperativeConfig.instance.callerLocation });
            return require(filePath);
        }
        catch (e) {
            // couldn't locate using find-up, try to require directly
            findupErr = e;
        }
        // if we couldn't find the file path through find-up, try requiring the string directly
        try {
            return require(file);
        }
        catch (e) {
            e.message = "Could not locate the specified module through requiring directly, nor through " +
                "searching the directories above " + this.callerLocation +
                ". 'require()' error message: " + e.message +
                " \n 'find-up' (directory search) error message:" + findupErr.message;
            throw new error_1.ImperativeError({ msg: e.message });
        }
    }
    /**
     * Append any auto generated commands to the root command document depending on configuration.
     * @param {ICommandDefinition} rootCommand - the root command as built so far
     * @returns {ICommandDefinition} - the root command with any auto generated commands appended
     */
    addAutoGeneratedCommands(rootCommand) {
        const loadedConfig = this.loadedConfig;
        if ((loadedConfig.autoGenerateProfileCommands == null || loadedConfig.autoGenerateProfileCommands) &&
            loadedConfig.profiles != null &&
            loadedConfig.profiles.length > 0) {
            rootCommand.children.push(CompleteProfilesGroupBuilder_1.CompleteProfilesGroupBuilder.getProfileGroup(loadedConfig.profiles, this.console));
        }
        return rootCommand;
    }
}
/**
 * This is the variable that store the specific instance of Imperative Config.
 * Defined as static so that it can be accessed from anywhere.
 *
 * @private
 * @type {ImperativeConfig}
 */
ImperativeConfig.mInstance = null;
exports.ImperativeConfig = ImperativeConfig;
//# sourceMappingURL=ImperativeConfig.js.map