import { ISettingsFile } from "./doc/ISettingsFile";
/**
 * A recovery function to handle when a settings file is missing on an initialization function.
 *
 * @param settingsFile The file that could not be found
 * @param defaultSettings The default settings provided by {@link AppSettings}
 *
 * @returns The settings content to be merged into the defaults.
 */
export declare type FileRecovery = (settingsFile: string, defaultSettings: ISettingsFile) => ISettingsFile;
/**
 * This class represents settings for an Imperative CLI application that can be configured
 * by an end user by modifying a settings file. Settings are stored in {@link AppSettings#settings}
 * in a format specified by {@link ISettingsFile}.
 */
export declare class AppSettings {
    private readonly settingsFile;
    /**
     *
     * @param settingsFile The settings file to load from.
     * @param missingFileRecovery A recovery function when the settings file isn't found
     *
     * @returns A reference to the created instance. This is the same instance returned by {@link AppSettings.instance}
     *
     * @throws {@link SettingsAlreadyInitialized} When the settings singleton has previously been initialized.
     */
    static initialize(settingsFile: string, missingFileRecovery?: FileRecovery): AppSettings;
    /**
     * This is an internal reference to the static settings instance.
     */
    private static mInstance;
    /**
     * Get the singleton instance of the app settings object that was initialized
     * within the {@link AppSettings.initialize} function.
     *
     * @returns A singleton AppSettings object
     *
     * @throws {@link SettingsNotInitialized} When the settings singleton has not been initialized.
     */
    static readonly instance: AppSettings;
    /**
     * The settings loaded from the file specified in the constructor.
     */
    readonly settings: ISettingsFile;
    /**
     * Constructs a new settings object
     *
     * @param settingsFile  The full path to a settings file to load.
     * @param missingFileRecovery A recovery function for when the settings file didn't exist
     */
    constructor(settingsFile: string, missingFileRecovery?: FileRecovery);
    /**
     * Set a new override and save it to the settings file.
     *
     * @param override The override being set
     * @param value The new value
     *
     * @returns A promise of completion
     */
    setNewOverride(override: keyof ISettingsFile["overrides"], value: string | false): Promise<void>;
    /**
     * @returns true if the app settings have been initialized
     */
    static readonly initialized: boolean;
    /**
     * Writes settings to the file.
     *
     * @returns A promise of completion
     */
    private writeSettingsFile;
}
