"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zowe = require("@brightside/core");
const utils_1 = require("./utils");
const vscode = require("vscode");
const ZoweUSSNode_1 = require("./ZoweUSSNode");
const Profiles_1 = require("./Profiles");
const PersistentFilters_1 = require("./PersistentFilters");
const extension = require("../src/extension");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Creates the USS tree that contains nodes of sessions and data sets
 *
 * @export
 */
function createUSSTree(log) {
    return __awaiter(this, void 0, void 0, function* () {
        const tree = new USSTree();
        yield tree.initialize(log);
        yield tree.addSession();
        return tree;
    });
}
exports.createUSSTree = createUSSTree;
/**
 * A tree that contains nodes of sessions and USS Files
 *
 * @export
 * @class USSTree
 * @implements {vscode.TreeDataProvider}
 */
class USSTree {
    constructor() {
        this.mFavorites = [];
        // Event Emitters used to notify subscribers that the refresh event has fired
        this.mOnDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this.mOnDidChangeTreeData.event;
        this.validProfile = -1;
        this.mSessionNodes = [];
        this.mFavoriteSession = new ZoweUSSNode_1.ZoweUSSNode(localize(0, null), vscode.TreeItemCollapsibleState.Collapsed, null, null, null);
        this.mFavoriteSession.contextValue = extension.FAVORITE_CONTEXT;
        this.mSessionNodes = [this.mFavoriteSession];
        this.mFavoriteSession.iconPath = utils_1.applyIcons(this.mFavoriteSession);
        this.mHistory = new PersistentFilters_1.PersistentFilters(USSTree.persistenceSchema);
    }
    /**
     * Takes argument of type ZoweUSSNode and returns it converted to a general [TreeItem]
     *
     * @param {ZoweUSSNode} element
     * @returns {vscode.TreeItem}
     */
    getTreeItem(element) {
        return element;
    }
    /**
     * Takes argument of type ZoweUSSNode and retrieves all of the first level children
     *
     * @param {ZoweUSSNode} [element] - Optional parameter; if not passed, returns root session nodes
     * @returns {ZoweUSSNode[] | Promise<ZoweUSSNode[]>}
     */
    getChildren(element) {
        if (element) {
            if (element.contextValue === extension.FAVORITE_CONTEXT) {
                return this.mFavorites;
            }
            return element.getChildren();
        }
        return this.mSessionNodes;
    }
    /**
     * Called whenever the tree needs to be refreshed, and fires the data change event
     *
     */
    refresh() {
        this.mOnDidChangeTreeData.fire();
    }
    /**
     * Called whenever the tree needs to be refreshed, and fires the data change event
     *
     */
    refreshElement(element) {
        element.dirty = true;
        this.mOnDidChangeTreeData.fire(element);
    }
    /**
     * Returns the parent node or null if it has no parent
     *
     * @param {ZoweUSSNode} element
     * @returns {vscode.ProviderResult<ZoweUSSNode>}
     */
    getParent(element) {
        return element.mParent;
    }
    /**
     * Adds a new session to the uss files tree
     *
     * @param {string} [sessionName] - optional; loads persisted profiles or default if not passed
     */
    addSession(sessionName) {
        return __awaiter(this, void 0, void 0, function* () {
            // Loads profile associated with passed sessionName, persisted profiles or default if none passed
            if (sessionName) {
                const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(sessionName);
                if (zosmfProfile) {
                    this.addSingleSession(zosmfProfile);
                }
            }
            else {
                const zosmfProfiles = Profiles_1.Profiles.getInstance().allProfiles;
                for (const zosmfProfile of zosmfProfiles) {
                    // If session is already added, do nothing
                    if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === zosmfProfile.name)) {
                        continue;
                    }
                    for (const session of this.mHistory.getSessions()) {
                        if (session === zosmfProfile.name) {
                            this.addSingleSession(zosmfProfile);
                        }
                    }
                }
                if (this.mSessionNodes.length === 1) {
                    this.addSingleSession(Profiles_1.Profiles.getInstance().defaultProfile);
                }
            }
            this.refresh();
        });
    }
    /**
     * Removes a session from the list in the uss files tree
     *
     * @param {ZoweUSSNode} [node]
     */
    deleteSession(node) {
        this.mSessionNodes = this.mSessionNodes.filter((tempNode) => tempNode.label.trim() !== node.label.trim());
        let revisedLabel = node.label;
        if (revisedLabel.includes("[")) {
            revisedLabel = revisedLabel.substring(0, revisedLabel.indexOf(" ["));
        }
        this.mHistory.removeSession(revisedLabel);
        this.refresh();
    }
    /**
     * Adds a node to the USS favorites list
     *
     * @param {ZoweUSSNode} node
     */
    addUSSFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let temp;
            temp = new ZoweUSSNode_1.ZoweUSSNode(node.fullPath, node.collapsibleState, this.mFavoriteSession, node.getSession(), node.mParent.fullPath, false, node.getSessionNode().mProfileName);
            temp.contextValue += extension.FAV_SUFFIX;
            if (temp.contextValue === extension.DS_TEXT_FILE_CONTEXT + extension.FAV_SUFFIX ||
                temp.contextValue === extension.DS_BINARY_FILE_CONTEXT + extension.FAV_SUFFIX) {
                temp.command = { command: "zowe.uss.ZoweUSSNode.open", title: "Open", arguments: [temp] };
            }
            temp.iconPath = utils_1.applyIcons(node);
            if (!this.mFavorites.find((tempNode) => tempNode.label === temp.label)) {
                this.mFavorites.push(temp);
                utils_1.sortTreeItems(this.mFavorites, extension.USS_SESSION_CONTEXT + extension.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Adds a search node to the USS favorites list
     *
     * @param {ZoweUSSNode} node
     */
    addUSSSearchFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const label = "[" + node.getSessionNode().mProfileName + "]: " + node.fullPath;
            const temp = new ZoweUSSNode_1.ZoweUSSNode(label, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, node.getSession(), null, false, node.getSessionNode().mProfileName);
            temp.fullPath = node.fullPath;
            temp.label = temp.tooltip = label;
            temp.contextValue = extension.USS_SESSION_CONTEXT + extension.FAV_SUFFIX;
            temp.iconPath = utils_1.applyIcons(temp);
            temp.command = { command: "zowe.uss.fullPath", title: "", arguments: [temp] };
            if (!this.mFavorites.find((tempNode) => tempNode.label === temp.label)) {
                this.mFavorites.push(temp);
                utils_1.sortTreeItems(this.mFavorites, extension.USS_SESSION_CONTEXT + extension.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Removes a node from the favorites list
     *
     * @param {ZoweUSSNode} node
     */
    removeUSSFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mFavorites = this.mFavorites.filter((temp) => !((temp.fullPath === node.fullPath) && (temp.contextValue.startsWith(node.contextValue))));
            yield this.updateFavorites();
            this.refreshElement(this.mFavoriteSession);
        });
    }
    updateFavorites() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = Object.assign({}, vscode.workspace.getConfiguration().get(USSTree.persistenceSchema));
            if (settings.persistence) {
                settings.favorites = this.mFavorites.map((fav) => (fav.fullPath.startsWith(fav.profileName) ? fav.fullPath : fav.profileName + fav.fullPath) + "{" +
                    fav.contextValue.substring(0, fav.contextValue.indexOf(extension.FAV_SUFFIX)) + "}");
                yield vscode.workspace.getConfiguration().update(USSTree.persistenceSchema, settings, vscode.ConfigurationTarget.Global);
            }
        });
    }
    /**
     * Change the state of an expandable node
     * @param provider the tree view provider
     * @param element the node being flipped
     * @param isOpen the intended state of the the tree view provider, true or false
     */
    flipState(element, isOpen = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (element.label !== "Favorites") {
                let usrNme;
                let passWrd;
                let baseEncd;
                if ((!element.getSession().ISession.user) || (!element.getSession().ISession.password)) {
                    try {
                        const values = yield Profiles_1.Profiles.getInstance().promptCredentials(element.mProfileName);
                        if (values !== undefined) {
                            usrNme = values[0];
                            passWrd = values[1];
                            baseEncd = values[2];
                        }
                    }
                    catch (error) {
                        yield utils_1.errorHandling(error, element.getProfileName(), localize(1, null) + `flipState.optionalProfiles!`);
                    }
                    if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                        element.getSession().ISession.user = usrNme;
                        element.getSession().ISession.password = passWrd;
                        element.getSession().ISession.base64EncodedAuth = baseEncd;
                        this.validProfile = 1;
                    }
                    else {
                        return;
                    }
                    yield this.refreshElement(element);
                    yield this.refresh();
                }
                else {
                    this.validProfile = 1;
                }
            }
            else {
                this.validProfile = 1;
            }
            if (this.validProfile === 1) {
                element.iconPath = utils_1.applyIcons(element, isOpen ? extension.ICON_STATE_OPEN : extension.ICON_STATE_CLOSED);
                element.dirty = true;
                this.mOnDidChangeTreeData.fire(element);
            }
        });
    }
    onDidChangeConfiguration(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (e.affectsConfiguration(USSTree.persistenceSchema)) {
                const setting = Object.assign({}, vscode.workspace.getConfiguration().get(USSTree.persistenceSchema));
                if (!setting.persistence) {
                    setting.favorites = [];
                    setting.history = [];
                    yield vscode.workspace.getConfiguration().update(USSTree.persistenceSchema, setting, vscode.ConfigurationTarget.Global);
                }
            }
        });
    }
    addHistory(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mHistory.addHistory(criteria);
            this.refresh();
        });
    }
    /**
     * Prompts the user for a path, and populates the [TreeView]{@link vscode.TreeView} based on the path
     *
     * @param {ZoweUSSNode} node - The session node
     * @returns {Promise<void>}
     */
    ussFilterPrompt(node) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.log) {
                this.log.debug(localize(2, null));
            }
            let sessionNode = node.getSessionNode();
            let remotepath;
            let usrNme;
            let passWrd;
            let baseEncd;
            if ((!(node.getSession().ISession.user).trim()) || (!(node.getSession().ISession.password).trim())) {
                try {
                    const values = yield Profiles_1.Profiles.getInstance().promptCredentials(node.mProfileName);
                    if (values !== undefined) {
                        usrNme = values[0];
                        passWrd = values[1];
                        baseEncd = values[2];
                    }
                }
                catch (error) {
                    yield utils_1.errorHandling(error, node.getProfileName(), localize(3, null) + `ussFilterPrompt.optionalProfiles!`);
                }
                if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                    node.getSession().ISession.user = usrNme;
                    node.getSession().ISession.password = passWrd;
                    node.getSession().ISession.base64EncodedAuth = baseEncd;
                    this.validProfile = 0;
                }
                else {
                    return;
                }
                yield this.refreshElement(node);
                yield this.refresh();
            }
            else {
                this.validProfile = 0;
            }
            if (this.validProfile === 0) {
                if (node.contextValue === extension.USS_SESSION_CONTEXT) {
                    if (this.mHistory.getHistory().length > 0) {
                        const createPick = new utils_1.FilterDescriptor(USSTree.defaultDialogText);
                        const items = this.mHistory.getHistory().map((element) => new utils_1.FilterItem(element));
                        if (extension.ISTHEIA) {
                            const options1 = {
                                placeHolder: localize(4, null)
                            };
                            // get user selection
                            const choice = (yield vscode.window.showQuickPick([createPick, ...items], options1));
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(5, null));
                                return;
                            }
                            remotepath = choice === createPick ? "" : choice.label;
                        }
                        else {
                            const quickpick = vscode.window.createQuickPick();
                            quickpick.placeholder = localize(6, null);
                            quickpick.items = [createPick, ...items];
                            quickpick.ignoreFocusOut = true;
                            quickpick.show();
                            const choice = yield utils_1.resolveQuickPickHelper(quickpick);
                            quickpick.hide();
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(7, null));
                                return;
                            }
                            if (choice instanceof utils_1.FilterDescriptor) {
                                if (quickpick.value) {
                                    remotepath = quickpick.value;
                                }
                            }
                            else {
                                remotepath = choice.label;
                            }
                        }
                    }
                    if (!remotepath) {
                        // manually entering a search - switch to an input box
                        const options = {
                            prompt: localize(8, null),
                            value: sessionNode.fullPath
                        };
                        // get user input
                        remotepath = yield vscode.window.showInputBox(options);
                        if (!remotepath || remotepath.length === 0) {
                            vscode.window.showInformationMessage(localize(9, null));
                            return;
                        }
                    }
                }
                else {
                    // executing search from saved search in favorites
                    remotepath = node.label.trim().substring(node.label.trim().indexOf(":") + 2);
                    const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
                    yield this.addSession(session);
                    const faveNode = node;
                    sessionNode = this.mSessionNodes.find((tempNode) => tempNode.mProfileName === session);
                    if ((!sessionNode.getSession().ISession.user) || (!sessionNode.getSession().ISession.password)) {
                        sessionNode.getSession().ISession.user = faveNode.getSession().ISession.user;
                        sessionNode.getSession().ISession.password = faveNode.getSession().ISession.password;
                        sessionNode.getSession().ISession.base64EncodedAuth = faveNode.getSession().ISession.base64EncodedAuth;
                    }
                }
                // Sanitization: Replace multiple preceding forward slashes with just one forward slash
                const sanitizedPath = remotepath.replace(/\/\/+/, "/");
                sessionNode.tooltip = sessionNode.fullPath = sanitizedPath;
                sessionNode.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                sessionNode.iconPath = utils_1.applyIcons(sessionNode, extension.ICON_STATE_OPEN);
                // update the treeview with the new path
                sessionNode.label = `${sessionNode.mProfileName} [${sanitizedPath}]`;
                sessionNode.dirty = true;
                this.addHistory(sanitizedPath);
            }
        });
    }
    initialize(log) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log = log;
            this.log.debug(localize(10, null));
            const favoriteSearchPattern = /^\[.+\]\:\s.*\{uss_session\}$/;
            const directorySearchPattern = /^\[.+\]\:\s.*\{directory\}$/;
            const lines = this.mHistory.readFavorites();
            lines.forEach((line) => {
                const profileName = line.substring(1, line.lastIndexOf("]"));
                const nodeName = (line.substring(line.indexOf(":") + 1, line.indexOf("{"))).trim();
                const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                try {
                    const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                    const session = zowe.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile);
                    let node;
                    if (directorySearchPattern.test(line)) {
                        node = new ZoweUSSNode_1.ZoweUSSNode(nodeName, vscode.TreeItemCollapsibleState.Collapsed, this.mFavoriteSession, session, "", false, profileName);
                    }
                    else if (favoriteSearchPattern.test(line)) {
                        const label = "[" + sesName + "]: " + nodeName;
                        node = new ZoweUSSNode_1.ZoweUSSNode(label, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, null, false, profileName);
                        node.contextValue = extension.USS_SESSION_CONTEXT;
                        node.fullPath = nodeName;
                        node.label = node.tooltip = label;
                        // add a command to execute the search
                        node.command = { command: "zowe.uss.fullPath", title: "", arguments: [node] };
                    }
                    else {
                        node = new ZoweUSSNode_1.ZoweUSSNode(nodeName, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, "", false, profileName);
                        node.command = { command: "zowe.uss.ZoweUSSNode.open",
                            title: localize(11, null), arguments: [node] };
                    }
                    node.contextValue += extension.FAV_SUFFIX;
                    node.iconPath = utils_1.applyIcons(node);
                    this.mFavorites.push(node);
                }
                catch (e) {
                    const errMessage = localize(12, null) + profileName +
                        localize(13, null) +
                        localize(14, null) +
                        utils_1.getAppName(extension.ISTHEIA) + localize(15, null);
                    utils_1.errorHandling(e, null, errMessage);
                    return;
                }
            });
        });
    }
    /**
     * Adds a single session to the USS tree
     *
     */
    addSingleSession(zosmfProfile) {
        return __awaiter(this, void 0, void 0, function* () {
            if (zosmfProfile) {
                // If session is already added, do nothing
                if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === zosmfProfile.name)) {
                    return;
                }
                // Uses loaded profile to create a zosmf session with brightside
                const session = zowe.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile);
                // Creates ZoweNode to track new session and pushes it to mSessionNodes
                const node = new ZoweUSSNode_1.ZoweUSSNode(zosmfProfile.name, vscode.TreeItemCollapsibleState.Collapsed, null, session, "", false, zosmfProfile.name);
                node.contextValue = extension.USS_SESSION_CONTEXT;
                node.iconPath = utils_1.applyIcons(node);
                node.dirty = true;
                this.mSessionNodes.push(node);
                this.mHistory.addSession(zosmfProfile.name);
            }
        });
    }
}
exports.USSTree = USSTree;
USSTree.defaultDialogText = "\uFF0B " + localize(16, null);
USSTree.persistenceSchema = "Zowe-USS-Persistent";

//# sourceMappingURL=USSTree.js.map
