"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zowe = require("@brightside/core");
const vscode = require("vscode");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
const extension = require("../src/extension");
const utils = require("./utils");
/**
 * A type of TreeItem used to represent sessions and USS directories and files
 *
 * @export
 * @class ZoweUSSNode
 * @extends {vscode.TreeItem}
 */
class ZoweUSSNode extends vscode.TreeItem {
    /**
     * Creates an instance of ZoweUSSNode
     *
     * @param {string} label - Displayed in the [TreeView]
     * @param {vscode.TreeItemCollapsibleState} collapsibleState - file/directory
     * @param {ZoweUSSNode} mParent - The parent node
     * @param {Session} session
     * @param {String} parentPath - The file path of the parent on the server
     * @param {String} mProfileName - Profile to which the node belongs to
     */
    constructor(label, collapsibleState, mParent, session, parentPath, binary = false, mProfileName, etag) {
        super(label, collapsibleState);
        this.mParent = mParent;
        this.session = session;
        this.parentPath = parentPath;
        this.binary = binary;
        this.mProfileName = mProfileName;
        this.etag = etag;
        this.fullPath = "";
        this.dirty = extension.ISTHEIA; // Make sure this is true for theia instances
        this.children = [];
        this.binaryFiles = {};
        this.profileName = "";
        this.shortLabel = "";
        if (collapsibleState !== vscode.TreeItemCollapsibleState.None) {
            this.contextValue = extension.USS_DIR_CONTEXT;
        }
        else if (binary) {
            this.contextValue = extension.DS_BINARY_FILE_CONTEXT + extension.FAV_SUFFIX;
        }
        else {
            this.contextValue = extension.DS_TEXT_FILE_CONTEXT;
        }
        if (this.parentPath) {
            this.fullPath = this.tooltip = this.parentPath + "/" + label;
            if (parentPath === "/") {
                // Keep fullPath of root level nodes preceded by a single slash
                this.fullPath = this.tooltip = "/" + label;
            }
        }
        if (this.mParent && this.mParent.contextValue === extension.FAVORITE_CONTEXT) {
            this.profileName = "[" + mProfileName + "]: ";
            this.fullPath = label.trim();
            // File or directory name only (no parent path)
            this.shortLabel = this.fullPath.split("/", this.fullPath.length).pop();
            // Display name for favorited file or directory in tree view
            this.label = this.profileName + this.shortLabel;
            this.tooltip = this.profileName + this.fullPath;
        }
        this.etag = etag ? etag : "";
        utils.applyIcons(this);
    }
    /**
     * Implements access to profile name
     * for {IZoweTreeNode}.
     *
     * @returns {string}
     */
    getProfileName() {
        return this.mProfileName;
    }
    /**
     * Retrieves child nodes of this ZoweUSSNode
     *
     * @returns {Promise<ZoweUSSNode[]>}
     */
    getChildren() {
        return __awaiter(this, void 0, void 0, function* () {
            if ((!this.fullPath && this.contextValue === extension.USS_SESSION_CONTEXT) ||
                (this.contextValue === extension.DS_TEXT_FILE_CONTEXT ||
                    this.contextValue === extension.DS_BINARY_FILE_CONTEXT + extension.FAV_SUFFIX)) {
                return [];
            }
            if (!this.dirty) {
                if (this.collapsibleState === vscode.TreeItemCollapsibleState.Collapsed) {
                    this.children = [];
                }
                return this.children;
            }
            if (!this.label) {
                vscode.window.showErrorMessage(localize(0, null));
                throw Error("Invalid node");
            }
            // Gets the directories from the fullPath and displays any thrown errors
            const responses = [];
            try {
                responses.push(yield vscode.window.withProgress({
                    location: vscode.ProgressLocation.Notification,
                    title: localize(1, null)
                }, () => {
                    return zowe.List.fileList(this.getSession(), this.fullPath);
                }));
            }
            catch (err) {
                utils.errorHandling(err, this.label, localize(2, null) + `zowe.List`);
            }
            // push nodes to an object with property names to avoid duplicates
            const elementChildren = {};
            responses.forEach((response) => {
                // Throws reject if the brightside command does not throw an error but does not succeed
                if (!response.success) {
                    throw Error(localize(3, null));
                }
                // Loops through all the returned file references members and creates nodes for them
                for (const item of response.apiResponse.items) {
                    const existing = this.children.find((element) => element.label.trim() === item.name);
                    if (existing) {
                        elementChildren[existing.label] = existing;
                    }
                    else if (item.name !== "." && item.name !== "..") {
                        // Creates a ZoweUSSNode for a directory
                        if (item.mode.startsWith("d")) {
                            const temp = new ZoweUSSNode(item.name, vscode.TreeItemCollapsibleState.Collapsed, this, null, this.fullPath, false, item.mProfileName);
                            elementChildren[temp.label] = temp;
                        }
                        else {
                            // Creates a ZoweUSSNode for a file
                            let temp;
                            if (this.getSessionNode().binaryFiles.hasOwnProperty(this.fullPath + "/" + item.name)) {
                                temp = new ZoweUSSNode(item.name, vscode.TreeItemCollapsibleState.None, this, null, this.fullPath, true, item.mProfileName);
                            }
                            else {
                                temp = new ZoweUSSNode(item.name, vscode.TreeItemCollapsibleState.None, this, null, this.fullPath, false, item.mProfileName);
                            }
                            temp.command = { command: "zowe.uss.ZoweUSSNode.open",
                                title: localize(4, null), arguments: [temp] };
                            elementChildren[temp.label] = temp;
                        }
                    }
                }
            });
            if (this.contextValue === extension.USS_SESSION_CONTEXT) {
                this.dirty = false;
            }
            return this.children = Object.keys(elementChildren).sort().map((labels) => elementChildren[labels]);
        });
    }
    /**
     * Returns the [Session] for this node
     *
     * @returns {Session}
     */
    getSession() {
        return this.session || this.mParent.getSession();
    }
    /**
     * Returns the session node for this node
     *
     * @returns {ZoweUSSNode}
     */
    getSessionNode() {
        return this.session ? this : this.mParent.getSessionNode();
    }
    setBinary(binary) {
        this.binary = binary;
        if (this.binary) {
            this.contextValue = extension.DS_BINARY_FILE_CONTEXT + extension.FAV_SUFFIX;
            this.getSessionNode().binaryFiles[this.fullPath] = true;
        }
        else {
            this.contextValue = extension.DS_TEXT_FILE_CONTEXT;
            delete this.getSessionNode().binaryFiles[this.fullPath];
        }
        if (this.mParent && this.mParent.contextValue === extension.FAVORITE_CONTEXT) {
            this.binary ? this.contextValue = extension.DS_BINARY_FILE_CONTEXT + extension.FAV_SUFFIX :
                this.contextValue = extension.DS_TEXT_FILE_CONTEXT + extension.FAV_SUFFIX;
        }
        utils.applyIcons(this);
        this.dirty = true;
    }
    /**
     * helper method to change the node names in one go
     * @param oldReference string
     * @param revision string
     */
    rename(newFullPath) {
        const oldReference = this.shortLabel;
        this.fullPath = newFullPath;
        this.shortLabel = newFullPath.substring(newFullPath.lastIndexOf("/"));
        this.label = this.label.replace(oldReference, this.shortLabel);
        this.tooltip = this.tooltip.replace(oldReference, this.shortLabel);
    }
    /**
     * Returns the [etag] for this node
     *
     * @returns {string}
     */
    getEtag() {
        return this.etag;
    }
    /**
     * Set the [etag] for this node
     *
     * @returns {void}
     */
    setEtag(etagValue) {
        this.etag = etagValue;
        /**
         * helper method to change the node names in one go
         * @param oldReference string
         * @param revision string
         */
    }
}
exports.ZoweUSSNode = ZoweUSSNode;

//# sourceMappingURL=ZoweUSSNode.js.map
