"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zowe = require("@brightside/core");
const fs = require("fs");
const os = require("os");
const fs_extra_1 = require("fs-extra");
const path = require("path");
const vscode = require("vscode");
const ZoweNode_1 = require("./ZoweNode");
const imperative_1 = require("@brightside/imperative");
const DatasetTree_1 = require("./DatasetTree");
const ZosJobsProvider_1 = require("./ZosJobsProvider");
const USSTree_1 = require("./USSTree");
const ussActions = require("./uss/ussNodeActions");
const mvsActions = require("./mvs/mvsNodeActions");
const MvsCommandHandler_1 = require("./command/MvsCommandHandler");
const Profiles_1 = require("./Profiles");
const nls = require("vscode-nls");
const utils = require("./utils");
const SpoolProvider_1 = require("./SpoolProvider");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
exports.ISTHEIA = false; // set during activate
exports.FAV_SUFFIX = "_fav";
exports.INFORMATION_CONTEXT = "information";
exports.FAVORITE_CONTEXT = "favorite";
exports.DS_FAV_CONTEXT = "ds_fav";
exports.PDS_FAV_CONTEXT = "pds_fav";
exports.DS_SESSION_CONTEXT = "session";
exports.DS_PDS_CONTEXT = "pds";
exports.DS_DS_CONTEXT = "ds";
exports.DS_MEMBER_CONTEXT = "member";
exports.DS_TEXT_FILE_CONTEXT = "textFile";
exports.DS_FAV_TEXT_FILE_CONTEXT = "textFile_fav";
exports.DS_BINARY_FILE_CONTEXT = "binaryFile";
exports.DS_MIGRATED_FILE_CONTEXT = "migr";
exports.USS_SESSION_CONTEXT = "uss_session";
exports.USS_DIR_CONTEXT = "directory";
exports.USS_FAV_DIR_CONTEXT = "directory_fav";
exports.JOBS_SESSION_CONTEXT = "server";
exports.JOBS_JOB_CONTEXT = "job";
exports.JOBS_SPOOL_CONTEXT = "spool";
exports.ICON_STATE_OPEN = "open";
exports.ICON_STATE_CLOSED = "closed";
let usrNme;
let passWrd;
let baseEncd;
let validProfile = -1;
let log;
/**
 * The function that runs when the extension is loaded
 *
 * @export
 * @param {vscode.ExtensionContext} context - Context of vscode at the time that the function is called
 */
function activate(context) {
    return __awaiter(this, void 0, void 0, function* () {
        // Get temp folder location from settings
        let preferencesTempPath = vscode.workspace.getConfiguration()
            /* tslint:disable:no-string-literal */
            .get("Zowe-Temp-Folder-Location")["folderPath"];
        // Determine the runtime framework to support special behavior for Theia
        const theia = "Eclipse Theia";
        const appName = vscode.env.appName;
        if (appName && appName === theia) {
            exports.ISTHEIA = true;
        }
        defineGlobals(preferencesTempPath);
        // Call cleanTempDir before continuing
        // this is to handle if the application crashed on a previous execution and
        // VSC didn't get a chance to call our deactivate to cleanup.
        yield deactivate();
        try {
            fs.mkdirSync(exports.BRIGHTTEMPFOLDER);
            fs.mkdirSync(exports.USS_DIR);
            fs.mkdirSync(exports.DS_DIR);
        }
        catch (err) {
            yield utils.errorHandling(err, null, err.message);
        }
        let datasetProvider;
        let ussFileProvider;
        let jobsProvider;
        try {
            // Initialize Imperative Logger
            const loggerConfig = require(path.join(context.extensionPath, "log4jsconfig.json"));
            loggerConfig.log4jsConfig.appenders.default.filename = path.join(context.extensionPath, "logs", "imperative.log");
            loggerConfig.log4jsConfig.appenders.imperative.filename = path.join(context.extensionPath, "logs", "imperative.log");
            loggerConfig.log4jsConfig.appenders.app.filename = path.join(context.extensionPath, "logs", "zowe.log");
            imperative_1.Logger.initLogger(loggerConfig);
            log = imperative_1.Logger.getAppLogger();
            log.debug(localize(0, null));
            const keytar = getSecurityModules("keytar");
            if (keytar) {
                const service = vscode.workspace.getConfiguration().get("Zowe Security: Credential Key");
                if (service) {
                    try {
                        // Override Imperative credential manager to use VSCode keytar
                        imperative_1.DefaultCredentialManager.prototype.initialize = () => __awaiter(this, void 0, void 0, function* () {
                            imperative_1.DefaultCredentialManager.prototype.keytar = keytar;
                        });
                        imperative_1.CredentialManagerFactory.initialize({
                            service
                        });
                    }
                    catch (err) {
                        throw new imperative_1.ImperativeError({ msg: err.toString() });
                    }
                }
            }
            yield Profiles_1.Profiles.createInstance(log);
            // Initialize dataset provider
            datasetProvider = yield DatasetTree_1.createDatasetTree(log);
            // Initialize uss provider
            ussFileProvider = yield USSTree_1.createUSSTree(log);
            // Initialize Jobs provider with the created session and the selected pattern
            jobsProvider = yield ZosJobsProvider_1.createJobsTree(log);
        }
        catch (err) {
            yield utils.errorHandling(err, null, (localize(1, null)));
            log.error(localize(2, null) + JSON.stringify(err));
        }
        const spoolProvider = new SpoolProvider_1.default();
        const providerRegistration = vscode.Disposable.from(vscode.workspace.registerTextDocumentContentProvider(SpoolProvider_1.default.scheme, spoolProvider));
        context.subscriptions.push(spoolProvider, providerRegistration);
        if (datasetProvider) {
            vscode.commands.registerCommand("zowe.addSession", () => __awaiter(this, void 0, void 0, function* () { return addZoweSession(datasetProvider); }));
            vscode.commands.registerCommand("zowe.addFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.addFavorite(node); }));
            vscode.commands.registerCommand("zowe.refreshAll", () => refreshAll(datasetProvider));
            vscode.commands.registerCommand("zowe.refreshNode", (node) => refreshPS(node));
            vscode.commands.registerCommand("zowe.pattern", (node) => datasetProvider.datasetFilterPrompt(node));
            vscode.commands.registerCommand("zowe.ZoweNode.openPS", (node) => openPS(node, true, datasetProvider));
            vscode.workspace.onDidSaveTextDocument((savedFile) => __awaiter(this, void 0, void 0, function* () {
                log.debug(localize(3, null) +
                    savedFile.fileName +
                    localize(4, null) +
                    exports.DS_DIR + localize(5, null) + exports.USS_DIR);
                if (savedFile.fileName.toUpperCase().indexOf(exports.DS_DIR.toUpperCase()) >= 0) {
                    log.debug(localize(6, null));
                    yield saveFile(savedFile, datasetProvider); // TODO MISSED TESTING
                }
                else if (savedFile.fileName.toUpperCase().indexOf(exports.USS_DIR.toUpperCase()) >= 0) {
                    log.debug(localize(7, null));
                    yield saveUSSFile(savedFile, ussFileProvider); // TODO MISSED TESTING
                }
                else {
                    log.debug(localize(8, null) + savedFile.fileName +
                        localize(9, null));
                }
            }));
            vscode.commands.registerCommand("zowe.createDataset", (node) => createFile(node, datasetProvider));
            vscode.commands.registerCommand("zowe.createMember", (node) => createMember(node, datasetProvider));
            vscode.commands.registerCommand("zowe.deleteDataset", (node) => deleteDataset(node, datasetProvider));
            vscode.commands.registerCommand("zowe.deletePDS", (node) => deleteDataset(node, datasetProvider));
            vscode.commands.registerCommand("zowe.uploadDialog", (node) => mvsActions.uploadDialog(node, datasetProvider));
            vscode.commands.registerCommand("zowe.deleteMember", (node) => deleteDataset(node, datasetProvider));
            vscode.commands.registerCommand("zowe.editMember", (node) => openPS(node, false, datasetProvider));
            vscode.commands.registerCommand("zowe.removeSession", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.deleteSession(node); }));
            vscode.commands.registerCommand("zowe.removeFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.removeFavorite(node); }));
            vscode.commands.registerCommand("zowe.saveSearch", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.addFavorite(node); }));
            vscode.commands.registerCommand("zowe.removeSavedSearch", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.removeFavorite(node); }));
            vscode.commands.registerCommand("zowe.submitJcl", () => __awaiter(this, void 0, void 0, function* () { return submitJcl(datasetProvider); }));
            vscode.commands.registerCommand("zowe.submitMember", (node) => __awaiter(this, void 0, void 0, function* () { return submitMember(node); }));
            vscode.commands.registerCommand("zowe.showDSAttributes", (node) => showDSAttributes(node, datasetProvider));
            vscode.commands.registerCommand("zowe.renameDataSet", (node) => renameDataSet(node, datasetProvider));
            vscode.commands.registerCommand("zowe.copyDataSet", (node) => copyDataSet(node));
            vscode.commands.registerCommand("zowe.pasteDataSet", (node) => pasteDataSet(node, datasetProvider));
            vscode.commands.registerCommand("zowe.renameDataSetMember", (node) => renameDataSetMember(node, datasetProvider));
            vscode.workspace.onDidChangeConfiguration((e) => __awaiter(this, void 0, void 0, function* () {
                datasetProvider.onDidChangeConfiguration(e);
            }));
            vscode.workspace.onDidChangeConfiguration((e) => {
                if (e.affectsConfiguration("Zowe-Temp-Folder-Location")) {
                    const updatedPreferencesTempPath = vscode.workspace.getConfiguration()
                        /* tslint:disable:no-string-literal */
                        .get("Zowe-Temp-Folder-Location")["folderPath"];
                    moveTempFolder(preferencesTempPath, updatedPreferencesTempPath);
                    // Update current temp folder preference
                    preferencesTempPath = updatedPreferencesTempPath;
                }
            });
            // Attaches the TreeView as a subscriber to the refresh event of datasetProvider
            const databaseView = vscode.window.createTreeView("zowe.explorer", { treeDataProvider: datasetProvider });
            context.subscriptions.push(databaseView);
            if (!exports.ISTHEIA) {
                databaseView.onDidCollapseElement((e) => __awaiter(this, void 0, void 0, function* () {
                    datasetProvider.flipState(e.element, false);
                }));
                databaseView.onDidExpandElement((e) => __awaiter(this, void 0, void 0, function* () {
                    datasetProvider.flipState(e.element, true);
                }));
            }
        }
        if (ussFileProvider) {
            vscode.commands.registerCommand("zowe.uss.addFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.addUSSFavorite(node); }));
            vscode.commands.registerCommand("zowe.uss.removeFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.removeUSSFavorite(node); }));
            vscode.commands.registerCommand("zowe.uss.addSession", () => __awaiter(this, void 0, void 0, function* () { return addZoweSession(ussFileProvider); }));
            vscode.commands.registerCommand("zowe.uss.refreshAll", () => ussActions.refreshAllUSS(ussFileProvider));
            vscode.commands.registerCommand("zowe.uss.refreshUSS", (node) => refreshUSS(node));
            vscode.commands.registerCommand("zowe.uss.fullPath", (node) => ussFileProvider.ussFilterPrompt(node));
            vscode.commands.registerCommand("zowe.uss.ZoweUSSNode.open", (node) => openUSS(node, false, true, ussFileProvider));
            vscode.commands.registerCommand("zowe.uss.removeSession", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.deleteSession(node); }));
            vscode.commands.registerCommand("zowe.uss.createFile", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.createUSSNode(node, ussFileProvider, "file"); }));
            vscode.commands.registerCommand("zowe.uss.createFolder", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.createUSSNode(node, ussFileProvider, "directory"); }));
            vscode.commands.registerCommand("zowe.uss.deleteNode", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.deleteUSSNode(node, ussFileProvider, getUSSDocumentFilePath(node)); }));
            vscode.commands.registerCommand("zowe.uss.binary", (node) => __awaiter(this, void 0, void 0, function* () { return changeFileType(node, true, ussFileProvider); }));
            vscode.commands.registerCommand("zowe.uss.text", (node) => __awaiter(this, void 0, void 0, function* () { return changeFileType(node, false, ussFileProvider); }));
            vscode.commands.registerCommand("zowe.uss.renameNode", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.renameUSSNode(node, ussFileProvider, getUSSDocumentFilePath(node)); }));
            vscode.commands.registerCommand("zowe.uss.uploadDialog", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.uploadDialog(node, ussFileProvider); }));
            vscode.commands.registerCommand("zowe.uss.createNode", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.createUSSNodeDialog(node, ussFileProvider); }));
            vscode.commands.registerCommand("zowe.uss.copyPath", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.copyPath(node); }));
            vscode.commands.registerCommand("zowe.uss.editFile", (node) => openUSS(node, false, false, ussFileProvider));
            vscode.commands.registerCommand("zowe.uss.saveSearch", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.addUSSSearchFavorite(node); }));
            vscode.commands.registerCommand("zowe.uss.removeSavedSearch", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.removeUSSFavorite(node); }));
            vscode.workspace.onDidChangeConfiguration((e) => __awaiter(this, void 0, void 0, function* () {
                ussFileProvider.onDidChangeConfiguration(e);
            }));
            const ussView = vscode.window.createTreeView("zowe.uss.explorer", { treeDataProvider: ussFileProvider });
            context.subscriptions.push(ussView);
            if (!exports.ISTHEIA) {
                ussView.onDidCollapseElement((e) => __awaiter(this, void 0, void 0, function* () {
                    ussFileProvider.flipState(e.element, false);
                }));
                ussView.onDidExpandElement((e) => __awaiter(this, void 0, void 0, function* () {
                    ussFileProvider.flipState(e.element, true);
                }));
            }
        }
        if (jobsProvider) {
            vscode.commands.registerCommand("zowe.zosJobsOpenspool", (session, spool) => {
                getSpoolContent(session, spool);
            });
            vscode.commands.registerCommand("zowe.deleteJob", (job) => jobsProvider.deleteJob(job));
            vscode.commands.registerCommand("zowe.runModifyCommand", (job) => {
                modifyCommand(job);
            });
            vscode.commands.registerCommand("zowe.runStopCommand", (job) => {
                stopCommand(job);
            });
            vscode.commands.registerCommand("zowe.refreshJobsServer", (node) => __awaiter(this, void 0, void 0, function* () { return refreshJobsServer(node, jobsProvider); }));
            vscode.commands.registerCommand("zowe.refreshAllJobs", () => {
                jobsProvider.mSessionNodes.forEach((jobNode) => {
                    if (jobNode.contextValue === exports.JOBS_SESSION_CONTEXT) {
                        jobNode.reset();
                    }
                });
                jobsProvider.refresh();
                Profiles_1.Profiles.getInstance().refresh();
            });
            vscode.commands.registerCommand("zowe.addJobsSession", () => addZoweSession(jobsProvider));
            vscode.commands.registerCommand("zowe.setOwner", (node) => {
                setOwner(node, jobsProvider);
            });
            vscode.commands.registerCommand("zowe.setPrefix", (node) => {
                setPrefix(node, jobsProvider);
            });
            vscode.commands.registerCommand("zowe.removeJobsSession", (node) => jobsProvider.deleteSession(node));
            vscode.commands.registerCommand("zowe.downloadSpool", (job) => downloadSpool(job));
            vscode.commands.registerCommand("zowe.getJobJcl", (job) => {
                downloadJcl(job);
            });
            vscode.commands.registerCommand("zowe.setJobSpool", (session, jobid) => __awaiter(this, void 0, void 0, function* () {
                const sessionNode = jobsProvider.mSessionNodes.find((jobNode) => {
                    return jobNode.label.trim() === session.trim();
                });
                sessionNode.dirty = true;
                jobsProvider.refresh();
                const jobs = yield sessionNode.getChildren();
                const job = jobs.find((jobNode) => {
                    return jobNode.job.jobid === jobid;
                });
                jobsProvider.setJob(jobView, job);
            }));
            vscode.commands.registerCommand("zowe.jobs.search", (node) => jobsProvider.searchPrompt(node));
            vscode.commands.registerCommand("zowe.issueTsoCmd", () => __awaiter(this, void 0, void 0, function* () { return MvsCommandHandler_1.MvsCommandHandler.getInstance().issueMvsCommand(); }));
            vscode.commands.registerCommand("zowe.issueMvsCmd", (node, command) => __awaiter(this, void 0, void 0, function* () { return MvsCommandHandler_1.MvsCommandHandler.getInstance().issueMvsCommand(node.session, command); }));
            vscode.workspace.onDidChangeConfiguration((e) => __awaiter(this, void 0, void 0, function* () {
                jobsProvider.onDidChangeConfiguration(e);
            }));
            vscode.commands.registerCommand("zowe.jobs.addFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.addJobsFavorite(node); }));
            vscode.commands.registerCommand("zowe.jobs.removeFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.removeJobsFavorite(node); }));
            vscode.commands.registerCommand("zowe.jobs.saveSearch", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.saveSearch(node); }));
            vscode.commands.registerCommand("zowe.jobs.removeSearchFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.removeJobsFavorite(node); }));
            const jobView = vscode.window.createTreeView("zowe.jobs", { treeDataProvider: jobsProvider });
            context.subscriptions.push(jobView);
            if (!exports.ISTHEIA) {
                jobView.onDidCollapseElement((e) => __awaiter(this, void 0, void 0, function* () {
                    jobsProvider.flipState(e.element, false);
                }));
                jobView.onDidExpandElement((e) => __awaiter(this, void 0, void 0, function* () {
                    jobsProvider.flipState(e.element, true);
                }));
            }
        }
    });
}
exports.activate = activate;
/**
 * Defines all global variables
 * @param tempPath File path for temporary folder defined in preferences
 */
function defineGlobals(tempPath) {
    tempPath !== "" && tempPath !== undefined ?
        exports.BRIGHTTEMPFOLDER = path.join(tempPath, "temp") :
        exports.BRIGHTTEMPFOLDER = path.join(__dirname, "..", "..", "resources", "temp");
    exports.USS_DIR = path.join(exports.BRIGHTTEMPFOLDER, "_U_");
    exports.DS_DIR = path.join(exports.BRIGHTTEMPFOLDER, "_D_");
}
exports.defineGlobals = defineGlobals;
/**
 * function to check if imperative.json contains
 * information about security or not and then
 * Imports the neccesary security modules
 */
function getSecurityModules(moduleName) {
    let imperativeIsSsecure = false;
    try {
        const fileName = path.join(getZoweDir(), "settings", "imperative.json");
        const settings = JSON.parse(fs.readFileSync(fileName).toString());
        const value1 = settings.overrides.CredentialManager;
        const value2 = settings.overrides["credential-manager"];
        imperativeIsSsecure = ((typeof value1 === "string") && (value1.length > 0)) ||
            ((typeof value2 === "string") && (value2.length > 0));
    }
    catch (error) {
        log.warn(localize(10, null) + error.message);
        vscode.window.showInformationMessage(error.message);
        return undefined;
    }
    if (imperativeIsSsecure) {
        // Workaround for Theia issue (https://github.com/eclipse-theia/theia/issues/4935)
        const appRoot = exports.ISTHEIA ? process.cwd() : vscode.env.appRoot;
        try {
            return require(`${appRoot}/node_modules/${moduleName}`);
        }
        catch (err) {
            vscode.window.showWarningMessage(localize(11, null) + moduleName);
        }
        try {
            return require(`${appRoot}/node_modules.asar/${moduleName}`);
        }
        catch (err) {
            vscode.window.showWarningMessage(localize(12, null) + moduleName);
        }
    }
    return undefined;
}
exports.getSecurityModules = getSecurityModules;
/**
 * Function to retrieve the home directory. In the situation Imperative has
 * not initialized it we mock a default value.
 */
function getZoweDir() {
    imperative_1.ImperativeConfig.instance.loadedConfig = {
        defaultHome: path.join(os.homedir(), ".zowe"),
        envVariablePrefix: "ZOWE"
    };
    const ti = imperative_1.ImperativeConfig.instance;
    return imperative_1.ImperativeConfig.instance.cliHome;
}
exports.getZoweDir = getZoweDir;
/**
 * Moves temp folder to user defined location in preferences
 * @param previousTempPath temp path settings value before updated by user
 * @param currentTempPath temp path settings value after updated by user
 */
function moveTempFolder(previousTempPath, currentTempPath) {
    // Re-define globals with updated path
    defineGlobals(currentTempPath);
    if (previousTempPath === "") {
        previousTempPath = path.join(__dirname, "..", "..", "resources");
    }
    // Make certain that "temp" folder is cleared
    cleanTempDir();
    try {
        fs.mkdirSync(exports.BRIGHTTEMPFOLDER);
        fs.mkdirSync(exports.USS_DIR);
        fs.mkdirSync(exports.DS_DIR);
    }
    catch (err) {
        log.error(localize(13, null) + JSON.stringify(err));
        utils.errorHandling(err, null, localize(14, null) + err.message);
    }
    const previousTemp = path.join(previousTempPath, "temp");
    try {
        // If source and destination path are same, exit
        if (previousTemp === exports.BRIGHTTEMPFOLDER) {
            return;
        }
        // TODO: Possibly remove when supporting "Multiple Instances"
        // If a second instance has already moved the temp folder, exit
        // Ideally, `moveSync()` would alert user if path doesn't exist.
        // However when supporting "Multiple Instances", might not be possible.
        if (!fs.existsSync(previousTemp)) {
            return;
        }
        fs_extra_1.moveSync(previousTemp, exports.BRIGHTTEMPFOLDER, { overwrite: true });
    }
    catch (err) {
        log.error("Error moving temporary folder! " + JSON.stringify(err));
        vscode.window.showErrorMessage(err.message);
    }
}
exports.moveTempFolder = moveTempFolder;
/**
 * Download all the spool content for the specified job.
 *
 * @param job The job to download the spool content from
 */
function downloadSpool(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const dirUri = yield vscode.window.showOpenDialog({
                openLabel: localize(15, null),
                canSelectFolders: true,
                canSelectFiles: false,
                canSelectMany: false
            });
            if (dirUri !== undefined) {
                zowe.DownloadJobs.downloadAllSpoolContentCommon(job.session, {
                    jobid: job.job.jobid,
                    jobname: job.job.jobname,
                    outDir: dirUri[0].fsPath
                });
            }
        }
        catch (error) {
            yield utils.errorHandling(error, null, error.message);
        }
    });
}
exports.downloadSpool = downloadSpool;
function downloadJcl(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const jobJcl = yield zowe.GetJobs.getJclForJob(job.session, job.job);
            const jclDoc = yield vscode.workspace.openTextDocument({ language: "jcl", content: jobJcl });
            yield vscode.window.showTextDocument(jclDoc);
        }
        catch (error) {
            yield utils.errorHandling(error, null, error.message);
        }
    });
}
exports.downloadJcl = downloadJcl;
/**
 * Switch the download type and redownload the file.
 *
 * @param node The file that is going to be downloaded
 * @param binary Whether the file should be downloaded as binary or not
 * @param ussFileProvider Our USSTree object
 */
function changeFileType(node, binary, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        node.setBinary(binary);
        yield openUSS(node, true, true, ussFileProvider);
        ussFileProvider.refresh();
    });
}
exports.changeFileType = changeFileType;
/**
 * Submit the contents of the editor as JCL.
 *
 * @export
 * @param {DatasetTree} datasetProvider - our DatasetTree object
 */
function submitJcl(datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const doc = vscode.window.activeTextEditor.document;
        log.debug(localize(16, null) + doc.fileName);
        // get session name
        const sessionregex = /\[(.*)(\])(?!.*\])/g;
        const regExp = sessionregex.exec(doc.fileName);
        const profiles = yield Profiles_1.Profiles.getInstance();
        let sesName;
        if (regExp === null) {
            const allProfiles = profiles.allProfiles;
            const profileNamesList = allProfiles.map((profile) => {
                return profile.name;
            });
            if (profileNamesList.length) {
                const quickPickOptions = {
                    placeHolder: localize(17, null),
                    ignoreFocusOut: true,
                    canPickMany: false
                };
                sesName = yield vscode.window.showQuickPick(profileNamesList, quickPickOptions);
            }
            else {
                vscode.window.showInformationMessage(localize(18, null));
            }
        }
        else {
            sesName = regExp[1];
            if (sesName.includes("[")) {
                // if submitting from favorites, sesName might be the favorite node, so extract further
                sesName = sessionregex.exec(sesName)[1];
            }
        }
        // get session from session name
        let documentSession;
        const sesNode = (yield datasetProvider.getChildren()).find((child) => child.label.trim() === sesName);
        if (sesNode) {
            documentSession = sesNode.getSession();
        }
        else {
            // if submitting from favorites, a session might not exist for this node
            const zosmfProfile = profiles.loadNamedProfile(sesName);
            documentSession = zowe.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile);
        }
        if (documentSession == null) {
            log.error(localize(19, null));
        }
        try {
            const job = yield zowe.SubmitJobs.submitJcl(documentSession, doc.getText());
            const args = [sesName, job.jobid];
            const setJobCmd = `command:zowe.setJobSpool?${encodeURIComponent(JSON.stringify(args))}`;
            vscode.window.showInformationMessage(localize(20, null) + `[${job.jobid}](${setJobCmd})`);
        }
        catch (error) {
            yield utils.errorHandling(error, sesName, localize(21, null) + error.message);
        }
    });
}
exports.submitJcl = submitJcl;
/**
 * Submit the selected dataset member as a Job.
 *
 * @export
 * @param node The dataset member
 */
function submitMember(node) {
    return __awaiter(this, void 0, void 0, function* () {
        const labelregex = /\[(.+)\]\: (.+)/g;
        let label;
        let sesName;
        switch (node.mParent.contextValue) {
            case (exports.FAVORITE_CONTEXT): {
                const regex = labelregex.exec(node.label);
                sesName = regex[1];
                label = regex[2];
                break;
            }
            case (exports.DS_PDS_CONTEXT + exports.FAV_SUFFIX): {
                const regex = labelregex.exec(node.mParent.label);
                sesName = regex[1];
                label = regex[2] + "(" + node.label.trim() + ")";
                break;
            }
            case (exports.DS_SESSION_CONTEXT):
                sesName = node.mParent.label;
                label = node.label;
                break;
            case (exports.DS_PDS_CONTEXT):
                sesName = node.mParent.mParent.label;
                label = node.mParent.label.trim() + "(" + node.label.trim() + ")";
                break;
            default:
                vscode.window.showErrorMessage(localize(22, null));
                throw Error(localize(23, null));
        }
        try {
            const job = yield zowe.SubmitJobs.submitJob(node.getSession(), label);
            const args = [sesName, job.jobid];
            const setJobCmd = `command:zowe.setJobSpool?${encodeURIComponent(JSON.stringify(args))}`;
            vscode.window.showInformationMessage(localize(24, null) + `[${job.jobid}](${setJobCmd})`);
        }
        catch (error) {
            yield utils.errorHandling(error, sesName, localize(25, null) + error.message);
        }
    });
}
exports.submitMember = submitMember;
/**
 * Adds a new Profile to the provided treeview by clicking the 'Plus' button and
 * selecting which profile you would like to add from the drop-down that appears.
 * The profiles that are in the tree view already will not appear in the
 * drop-down.
 *
 * @export
 * @param {USSTree} zoweFileProvider - either the USS, MVS, JES tree
 */
function addZoweSession(zoweFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const allProfiles = (yield Profiles_1.Profiles.getInstance()).allProfiles;
        const createNewProfile = "Create a New Connection to z/OS";
        let chosenProfile;
        let profileNamesList = allProfiles.map((profile) => {
            return profile.name;
        });
        if (profileNamesList) {
            profileNamesList = profileNamesList.filter((profileNames) => 
            // Find all cases where a profile is not already displayed
            !zoweFileProvider.mSessionNodes.find((sessionNode) => sessionNode.getProfileName() === profileNames));
        }
        const createPick = new utils.FilterDescriptor("\uFF0B " + createNewProfile);
        const items = profileNamesList.map((element) => new utils.FilterItem(element));
        const placeholder = localize(26, null);
        if (exports.ISTHEIA) {
            const options = {
                placeHolder: placeholder
            };
            // get user selection
            const choice = (yield vscode.window.showQuickPick([createPick, ...items], options));
            if (!choice) {
                vscode.window.showInformationMessage(localize(27, null));
                return;
            }
            chosenProfile = choice === createPick ? "" : choice.label;
        }
        else {
            const quickpick = vscode.window.createQuickPick();
            quickpick.items = [createPick, ...items];
            quickpick.placeholder = placeholder;
            quickpick.ignoreFocusOut = true;
            quickpick.show();
            const choice = yield utils.resolveQuickPickHelper(quickpick);
            quickpick.hide();
            if (!choice) {
                vscode.window.showInformationMessage(localize(28, null));
                return;
            }
            if (choice instanceof utils.FilterDescriptor) {
                chosenProfile = "";
            }
            else {
                chosenProfile = choice.label;
            }
        }
        if (chosenProfile === "") {
            let newprofile;
            let profileName;
            const options = {
                placeHolder: localize(29, null),
                prompt: localize(30, null),
                value: profileName
            };
            profileName = yield vscode.window.showInputBox(options);
            if (!profileName) {
                vscode.window.showInformationMessage(localize(31, null));
                return;
            }
            chosenProfile = profileName;
            log.debug(localize(32, null));
            try {
                newprofile = yield Profiles_1.Profiles.getInstance().createNewConnection(chosenProfile);
            }
            catch (error) {
                yield utils.errorHandling(error, chosenProfile, error.message);
            }
            if (newprofile) {
                try {
                    yield Profiles_1.Profiles.getInstance().refresh();
                }
                catch (error) {
                    yield utils.errorHandling(error, newprofile, error.message);
                }
                yield zoweFileProvider.addSession(newprofile);
                yield zoweFileProvider.refresh();
            }
        }
        else if (chosenProfile) {
            log.debug(localize(33, null) + chosenProfile);
            yield zoweFileProvider.addSession(chosenProfile);
        }
        else {
            log.debug(localize(34, null));
        }
    });
}
exports.addZoweSession = addZoweSession;
/**
 * Creates a new file and uploads to the server
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * TODO: Consider changing configuration to allow "custom" data set specifications
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * @export
 * @param {ZoweNode} node - Desired Brightside session
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function createFile(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const quickPickOptions = {
            placeHolder: localize(35, null),
            ignoreFocusOut: true,
            canPickMany: false
        };
        const types = [
            localize(36, null),
            localize(37, null),
            localize(38, null),
            localize(39, null),
            localize(40, null)
        ];
        let sesNamePrompt;
        if (node.contextValue.endsWith(exports.FAV_SUFFIX)) {
            sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
        }
        else {
            sesNamePrompt = node.label;
        }
        if ((!node.getSession().ISession.user) || (!node.getSession().ISession.password)) {
            try {
                const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                if (values !== undefined) {
                    usrNme = values[0];
                    passWrd = values[1];
                    baseEncd = values[2];
                }
            }
            catch (error) {
                yield utils.errorHandling(error, node.getProfileName(), error.message);
            }
            if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                node.getSession().ISession.user = usrNme;
                node.getSession().ISession.password = passWrd;
                node.getSession().ISession.base64EncodedAuth = baseEncd;
                validProfile = 0;
            }
            else {
                return;
            }
            yield datasetProvider.refreshElement(node);
            yield datasetProvider.refresh();
        }
        else {
            validProfile = 0;
        }
        if (validProfile === 0) {
            // get data set type
            const type = yield vscode.window.showQuickPick(types, quickPickOptions);
            if (type == null) {
                log.debug(localize(41, null));
                return;
            }
            else {
                log.debug(localize(42, null));
            }
            let typeEnum;
            let createOptions;
            switch (type) {
                case localize(43, null):
                    typeEnum = 0 /* DATA_SET_BINARY */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-Binary");
                    break;
                case localize(44, null):
                    typeEnum = 1 /* DATA_SET_C */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-C");
                    break;
                case localize(45, null):
                    typeEnum = 2 /* DATA_SET_CLASSIC */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-Classic");
                    break;
                case localize(46, null):
                    typeEnum = 3 /* DATA_SET_PARTITIONED */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-PDS");
                    break;
                case localize(47, null):
                    typeEnum = 4 /* DATA_SET_SEQUENTIAL */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-PS");
                    break;
            }
            // get name of data set
            let name = yield vscode.window.showInputBox({ placeHolder: localize(48, null) });
            name = name.toUpperCase();
            try {
                yield zowe.Create.dataSet(node.getSession(), typeEnum, name, createOptions);
                node.dirty = true;
                // Store previous filters (before refreshing)
                const currChildren = yield node.getChildren();
                let theFilter = datasetProvider.getHistory()[0] || null;
                // Check if filter is currently applied
                if (currChildren[0].contextValue !== "information" && theFilter) {
                    let addNewFilter = true;
                    const currentFilters = theFilter.split(", ");
                    // Check if current filter includes the new node
                    currentFilters.forEach((filter) => {
                        const regex = new RegExp(filter.replace(`*`, `(.+)`) + "$");
                        addNewFilter = regex.test(name) ? false : addNewFilter;
                    });
                    if (addNewFilter) {
                        theFilter = `${theFilter}, ${name}`;
                        datasetProvider.addHistory(theFilter);
                    }
                }
                else {
                    // No filter is currently applied
                    theFilter = name;
                    datasetProvider.addHistory(theFilter);
                }
                datasetProvider.refresh();
                // Show newly-created data set in expanded tree view
                if (name) {
                    node.label = `${node.label} `;
                    node.label = node.label.trim();
                    node.tooltip = node.pattern = theFilter.toUpperCase();
                    node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                    node.iconPath = utils.applyIcons(node, exports.ICON_STATE_OPEN);
                    node.dirty = true;
                    const newNode = yield node.getChildren().then((children) => children.find((child) => child.label === name));
                    const newNodeView = vscode.window.createTreeView("zowe.explorer", { treeDataProvider: datasetProvider });
                    newNodeView.reveal(newNode, { select: true });
                }
            }
            catch (err) {
                log.error(localize(49, null) + JSON.stringify(err));
                yield utils.errorHandling(err, node.getProfileName(), localize(50, null) +
                    err.message);
                throw (err);
            }
        }
    });
}
exports.createFile = createFile;
/**
 * Creates a PDS member
 *
 * @export
 * @param {ZoweNode} parent - The parent Node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function createMember(parent, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const name = yield vscode.window.showInputBox({ placeHolder: localize(51, null) });
        log.debug(localize(52, null) + name);
        if (name) {
            let label = parent.label.trim();
            if (parent.contextValue === exports.DS_PDS_CONTEXT + exports.FAV_SUFFIX) {
                label = parent.label.substring(parent.label.indexOf(":") + 2); // TODO MISSED TESTING
            }
            try {
                yield zowe.Upload.bufferToDataSet(parent.getSession(), Buffer.from(""), label + "(" + name + ")");
            }
            catch (err) {
                log.error(localize(53, null) + JSON.stringify(err));
                yield utils.errorHandling(err, label, localize(54, null) + err.message);
                throw (err);
            }
            parent.dirty = true;
            datasetProvider.refreshElement(parent);
            openPS(new ZoweNode_1.ZoweNode(name, vscode.TreeItemCollapsibleState.None, parent, null), true, datasetProvider);
            datasetProvider.refresh();
        }
    });
}
exports.createMember = createMember;
/**
 * Shows data set attributes in a new text editor
 *
 * @export
 * @param {ZoweNode} parent - The parent Node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function showDSAttributes(parent, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let label = parent.label.trim();
        if (parent.contextValue === exports.DS_PDS_CONTEXT + exports.FAV_SUFFIX || parent.contextValue === exports.DS_DS_CONTEXT + exports.FAV_SUFFIX) {
            label = parent.label.trim().substring(parent.label.trim().indexOf(":") + 2);
        }
        log.debug(localize(55, null) + label);
        let attributes;
        try {
            attributes = yield zowe.List.dataSet(parent.getSession(), label, { attributes: true });
            attributes = attributes.apiResponse.items;
            attributes = attributes.filter((dataSet) => {
                return dataSet.dsname.toUpperCase() === label.toUpperCase();
            });
            if (attributes.length === 0) {
                throw new Error(localize(56, null) + label);
            }
        }
        catch (err) {
            log.error(localize(57, null) + JSON.stringify(err));
            yield utils.errorHandling(err, parent.getProfileName(), localize(58, null) + err.message);
            throw (err);
        }
        // shouldn't be possible for there to be two cataloged data sets with the same name,
        // but just in case we'll display all of the results
        // if there's only one result (which there should be), we will just pass in attributes[0]
        // so that prettyJson doesn't display the attributes as an array with a hyphen character
        const attributesText = imperative_1.TextUtils.prettyJson(attributes.length > 1 ? attributes : attributes[0], undefined, false);
        // const attributesFilePath = path.join(BRIGHTTEMPFOLDER, label + ".yaml");
        // fs.writeFileSync(attributesFilePath, attributesText);
        // const document = await vscode.workspace.openTextDocument(attributesFilePath);
        // await vscode.window.showTextDocument(document);
        const attributesMessage = localize(59, null);
        const webviewHTML = `<!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <title>${label} "${attributesMessage}"</title>
    </head>
    <body>
     ${attributesText.replace(/\n/g, "</br>")}
    </body>
    </html>`;
        const column = vscode.window.activeTextEditor
            ? vscode.window.activeTextEditor.viewColumn
            : undefined;
        const panel = vscode.window.createWebviewPanel("zowe", label + " " + localize(60, null), column || 1, {});
        panel.webview.html = webviewHTML;
    });
}
exports.showDSAttributes = showDSAttributes;
/**
 * Rename data sets
 *
 * @export
 * @param {ZoweNode} node - The node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function renameDataSet(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let beforeDataSetName = node.label.trim();
        let favPrefix;
        let isFavourite;
        if (node.contextValue.includes(exports.FAV_SUFFIX)) {
            isFavourite = true;
            favPrefix = node.label.substring(0, node.label.indexOf(":") + 2);
            beforeDataSetName = node.label.substring(node.label.indexOf(":") + 2);
        }
        const afterDataSetName = yield vscode.window.showInputBox({ value: beforeDataSetName });
        log.debug(localize(61, null) + afterDataSetName);
        if (afterDataSetName) {
            try {
                yield zowe.Rename.dataSet(node.getSession(), beforeDataSetName, afterDataSetName);
                node.label = `${favPrefix}${afterDataSetName}`;
            }
            catch (err) {
                log.error(localize(62, null) + JSON.stringify(err));
                yield utils.errorHandling(err, favPrefix, localize(63, null) + err.message);
                throw err;
            }
            if (isFavourite) {
                const profile = favPrefix.substring(1, favPrefix.indexOf("]"));
                datasetProvider.renameNode(profile, beforeDataSetName, afterDataSetName);
            }
            else {
                const temp = node.label;
                node.label = "[" + node.getSessionNode().label.trim() + "]: " + beforeDataSetName;
                datasetProvider.renameFavorite(node, afterDataSetName);
                node.label = temp;
            }
            datasetProvider.refreshElement(node.mParent);
            datasetProvider.updateFavorites();
        }
    });
}
exports.renameDataSet = renameDataSet;
function getProfileAndDataSetName(node) {
    let profileName;
    let dataSetName;
    if (node.contextValue.includes(exports.FAV_SUFFIX)) {
        profileName = node.label.substring(1, node.label.indexOf("]"));
        dataSetName = node.label.substring(node.label.indexOf(":") + 2);
    }
    else {
        profileName = node.mParent.label.trim();
        dataSetName = node.label.trim();
    }
    return { profileName, dataSetName };
}
function getNodeLabels(node) {
    if (node.contextValue.includes(exports.DS_MEMBER_CONTEXT)) {
        return Object.assign(Object.assign({}, getProfileAndDataSetName(node.mParent)), { memberName: node.label.trim() });
    }
    else {
        return getProfileAndDataSetName(node);
    }
}
/**
 * Copy data sets
 *
 * @export
 * @param {ZoweNode} node - The node to copy
 */
function copyDataSet(node) {
    return __awaiter(this, void 0, void 0, function* () {
        return vscode.env.clipboard.writeText(JSON.stringify(getNodeLabels(node)));
    });
}
exports.copyDataSet = copyDataSet;
/**
 * Paste data sets
 *
 * @export
 * @param {ZoweNode} node - The node to paste to
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function pasteDataSet(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const { profileName, dataSetName } = getNodeLabels(node);
        let memberName;
        let beforeDataSetName;
        let beforeProfileName;
        let beforeMemberName;
        if (node.contextValue.includes(exports.DS_PDS_CONTEXT)) {
            memberName = yield vscode.window.showInputBox({ placeHolder: localize(64, null) });
            if (!memberName) {
                return;
            }
        }
        try {
            ({
                dataSetName: beforeDataSetName,
                memberName: beforeMemberName,
                profileName: beforeProfileName,
            } = JSON.parse(yield vscode.env.clipboard.readText()));
        }
        catch (err) {
            throw Error("Invalid clipboard. Copy from data set first");
        }
        if (beforeProfileName === profileName) {
            if (memberName) {
                try {
                    yield zowe.Get.dataSet(node.getSession(), `${dataSetName}(${memberName})`);
                    throw Error(`${dataSetName}(${memberName}) already exists. You cannot replace a member`);
                }
                catch (err) {
                    if (!err.message.includes("Member not found")) {
                        throw err;
                    }
                }
            }
            yield zowe.Copy.dataSet(node.getSession(), { dataSetName: beforeDataSetName, memberName: beforeMemberName }, { dataSetName, memberName });
            if (memberName) {
                datasetProvider.refreshElement(node);
                let node2;
                if (node.contextValue.includes(exports.FAV_SUFFIX)) {
                    node2 = datasetProvider.findNonFavoritedNode(node);
                }
                else {
                    node2 = datasetProvider.findFavoritedNode(node);
                }
                if (node2) {
                    datasetProvider.refreshElement(node2);
                }
            }
            else {
                refreshPS(node);
            }
        }
    });
}
exports.pasteDataSet = pasteDataSet;
/**
 * Rename data set members
 *
 * @export
 * @param {ZoweNode} node - The node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function renameDataSetMember(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const beforeMemberName = node.label.trim();
        let dataSetName;
        let profileLabel;
        if (node.mParent.contextValue.includes(exports.FAV_SUFFIX)) {
            profileLabel = node.mParent.label.substring(0, node.mParent.label.indexOf(":") + 2);
            dataSetName = node.mParent.label.substring(node.mParent.label.indexOf(":") + 2);
        }
        else {
            dataSetName = node.mParent.label.trim();
        }
        const afterMemberName = yield vscode.window.showInputBox({ value: beforeMemberName });
        log.debug(localize(65, null) + afterMemberName);
        if (afterMemberName) {
            try {
                yield zowe.Rename.dataSetMember(node.getSession(), dataSetName, beforeMemberName, afterMemberName);
                node.label = `${profileLabel}${afterMemberName}`;
            }
            catch (err) {
                log.error(localize(66, null) + JSON.stringify(err));
                yield utils.errorHandling(err, profileLabel, localize(67, null) + err.message);
                throw err;
            }
            if (node.mParent.contextValue.includes(exports.FAV_SUFFIX)) {
                const nonFavoritedParent = datasetProvider.findNonFavoritedNode(node.mParent);
                if (nonFavoritedParent) {
                    const nonFavoritedMember = nonFavoritedParent.children.find((child) => child.label === beforeMemberName);
                    if (nonFavoritedMember) {
                        nonFavoritedMember.label = afterMemberName;
                        datasetProvider.refreshElement(nonFavoritedParent);
                    }
                }
            }
            else {
                const favoritedParent = datasetProvider.findFavoritedNode(node.mParent);
                if (favoritedParent) {
                    const favoritedMember = favoritedParent.children.find((child) => child.label === beforeMemberName);
                    if (favoritedMember) {
                        favoritedMember.label = afterMemberName;
                        datasetProvider.refreshElement(favoritedParent);
                    }
                }
            }
            datasetProvider.refreshElement(node.mParent);
        }
    });
}
exports.renameDataSetMember = renameDataSetMember;
/**
 * Recursively deletes directory
 *
 * @param directory path to directory to be deleted
 */
function cleanDir(directory) {
    if (!fs.existsSync(directory)) {
        return;
    }
    fs.readdirSync(directory).forEach((file) => {
        const fullpath = path.join(directory, file);
        const lstat = fs.lstatSync(fullpath);
        if (lstat.isFile()) {
            fs.unlinkSync(fullpath);
        }
        else {
            cleanDir(fullpath);
        }
    });
    fs.rmdirSync(directory);
}
exports.cleanDir = cleanDir;
/**
 * Cleans up local temp directory
 *
 * @export
 */
function cleanTempDir() {
    return __awaiter(this, void 0, void 0, function* () {
        // logger hasn't necessarily been initialized yet, don't use the `log` in this function
        if (!fs.existsSync(exports.BRIGHTTEMPFOLDER)) {
            return;
        }
        try {
            cleanDir(exports.BRIGHTTEMPFOLDER);
        }
        catch (err) {
            vscode.window.showErrorMessage(localize(68, null) + err);
        }
    });
}
exports.cleanTempDir = cleanTempDir;
/**
 * Called by VSCode on shutdown
 *
 * @export
 */
function deactivate() {
    return __awaiter(this, void 0, void 0, function* () {
        yield cleanTempDir();
    });
}
exports.deactivate = deactivate;
/**
 * Deletes a dataset
 *
 * @export
 * @param {ZoweNode} node - The node to be deleted
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function deleteDataset(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        log.debug(localize(69, null) + node.label);
        const quickPickOptions = {
            placeHolder: localize(70, null) + node.label,
            ignoreFocusOut: true,
            canPickMany: false
        };
        // confirm that the user really wants to delete
        if ((yield vscode.window.showQuickPick([localize(71, null),
            localize(72, null)], quickPickOptions)) !== localize(73, null)) {
            log.debug(localize(74, null));
            return;
        }
        let label = "";
        let fav = false;
        try {
            switch (node.mParent.contextValue) {
                case (exports.FAVORITE_CONTEXT):
                    label = node.label.substring(node.label.indexOf(":") + 1).trim();
                    fav = true;
                    break;
                case (exports.DS_PDS_CONTEXT + exports.FAV_SUFFIX):
                    label = node.mParent.label.substring(node.mParent.label.indexOf(":") + 1).trim() + "(" + node.label.trim() + ")";
                    fav = true;
                    break;
                case (exports.DS_SESSION_CONTEXT):
                    label = node.label.trim();
                    break;
                case (exports.DS_PDS_CONTEXT):
                    label = node.mParent.label.trim() + "(" + node.label.trim() + ")";
                    break;
                default:
                    throw Error(localize(75, null));
            }
            yield zowe.Delete.dataSet(node.getSession(), label);
        }
        catch (err) {
            log.error(localize(76, null) + JSON.stringify(err));
            if (err.message.includes(localize(77, null))) {
                vscode.window.showInformationMessage(localize(78, null) + label +
                    localize(79, null));
            }
            else {
                yield utils.errorHandling(err, node.getProfileName(), err.message);
            }
            throw err;
        }
        // remove node from tree
        if (fav) {
            datasetProvider.mSessionNodes.forEach((ses) => {
                if (node.label.substring(node.label.indexOf("[") + 1, node.label.indexOf("]")) === ses.label.trim() ||
                    node.mParent.label.substring(node.mParent.label.indexOf("["), node.mParent.label.indexOf("]")) === ses.label) {
                    ses.dirty = true;
                }
            });
            datasetProvider.removeFavorite(node);
        }
        else {
            node.getSessionNode().dirty = true;
            const temp = node.label;
            node.label = "[" + node.getSessionNode().label.trim() + "]: " + node.label;
            datasetProvider.removeFavorite(node);
            node.label = temp;
        }
        datasetProvider.refresh();
        // remove local copy of file
        const fileName = getDocumentFilePath(label, node);
        try {
            if (fs.existsSync(fileName)) {
                fs.unlinkSync(fileName);
            }
        }
        catch (err) {
            // do nothing
        }
    });
}
exports.deleteDataset = deleteDataset;
/**
 * Prompts the user for a pattern, and populates the [TreeView]{@link vscode.TreeView} based on the pattern
 *
 * @param {ZoweNode} node - The session node
 * @param {DatasetTree} datasetProvider - Current DatasetTree used to populate the TreeView
 * @returns {Promise<void>}
 */
function enterPattern(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        if (log) {
            log.debug(localize(80, null));
        }
        let pattern;
        if (node.contextValue === exports.DS_SESSION_CONTEXT) {
            // manually entering a search
            const options = {
                prompt: localize(81, null),
                value: node.pattern
            };
            // get user input
            pattern = yield vscode.window.showInputBox(options);
            if (!pattern) {
                vscode.window.showInformationMessage(localize(82, null));
                return;
            }
        }
        else {
            // executing search from saved search in favorites
            pattern = node.label.trim().substring(node.label.trim().indexOf(":") + 2);
            const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
            yield datasetProvider.addSession(session);
            node = datasetProvider.mSessionNodes.find((tempNode) => tempNode.label.trim() === session);
        }
        // update the treeview with the new pattern
        // TODO figure out why a label change is needed to refresh the treeview,
        // instead of changing the collapsible state
        // change label so the treeview updates
        node.label = node.label.trim() + " ";
        node.label.trim();
        node.tooltip = node.pattern = pattern.toUpperCase();
        node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
        node.dirty = true;
        node.iconPath = utils.applyIcons(node, exports.ICON_STATE_OPEN);
        datasetProvider.addHistory(node.pattern);
    });
}
exports.enterPattern = enterPattern;
/**
 * Returns the profile for the specified node
 *
 * @export
 * @param {ZoweNode} node
 */
function getProfile(node) {
    let profile = node.getSessionNode().label.trim();
    // if this is a favorite node, further extraction is necessary
    if (profile.includes("[")) {
        profile = profile.substring(profile.indexOf("[") + 1, profile.indexOf("]"));
    }
    return profile;
}
exports.getProfile = getProfile;
/**
 * Returns the profile for the specified node
 *
 * @export
 * @param {ZoweUSSNode} node
 */
function getUSSProfile(node) {
    const profile = node.getSessionNode().mProfileName;
    return profile;
}
exports.getUSSProfile = getUSSProfile;
/**
 * Append a suffix on a ds file so it can be interpretted with syntax highlighter
 *
 * Rules of mapping:
 *  1. Start with LLQ and work backwards as it is at this end usually
 *   the language is specified
 *  2. Dont do this for the top level HLQ
 */
function appendSuffix(label) {
    const limit = 5;
    const bracket = label.indexOf("(");
    const split = (bracket > -1) ? label.substr(0, bracket).split(".", limit) : label.split(".", limit);
    for (let i = split.length - 1; i > 0; i--) {
        if (["JCL", "CNTL"].includes(split[i])) {
            return label.concat(".jcl");
        }
        if (["COBOL", "CBL", "COB", "SCBL"].includes(split[i])) {
            return label.concat(".cbl");
        }
        if (["COPYBOOK", "COPY", "CPY", "COBCOPY"].includes(split[i])) {
            return label.concat(".cpy");
        }
        if (["INC", "INCLUDE", "PLINC"].includes(split[i])) {
            return label.concat(".inc");
        }
        if (["PLI", "PL1", "PLX", "PCX"].includes(split[i])) {
            return label.concat(".pli");
        }
        if (["SH", "SHELL"].includes(split[i])) {
            return label.concat(".sh");
        }
        if (["REXX", "REXEC", "EXEC"].includes(split[i])) {
            return label.concat(".rexx");
        }
        if (split[i] === "XML") {
            return label.concat(".xml");
        }
        if (split[i] === "ASM" || split[i].indexOf("ASSEMBL") > -1) {
            return label.concat(".asm");
        }
        if (split[i] === "LOG" || split[i].indexOf("SPFLOG") > -1) {
            return label.concat(".log");
        }
    }
    return label;
}
/**
 * Returns the file path for the ZoweNode
 *
 * @export
 * @param {string} label - If node is a member, label includes the name of the PDS
 * @param {ZoweNode} node
 */
function getDocumentFilePath(label, node) {
    return path.join(exports.DS_DIR, "/" + getProfile(node) + "/" + appendSuffix(label));
}
exports.getDocumentFilePath = getDocumentFilePath;
/**
 * Returns the local file path for the ZoweUSSNode
 *
 * @export
 * @param {ZoweUSSNode} node
 */
function getUSSDocumentFilePath(node) {
    return path.join(exports.USS_DIR, "/" + getUSSProfile(node) + "/", node.fullPath);
}
exports.getUSSDocumentFilePath = getUSSDocumentFilePath;
/**
 * Downloads and displays a PS in a text editor view
 *
 * @param {ZoweNode} node
 */
function openPS(node, previewMember, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let sesNamePrompt;
        if (node.contextValue.endsWith(exports.FAV_SUFFIX)) {
            sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
        }
        else {
            sesNamePrompt = node.label;
        }
        if ((!node.getSession().ISession.user) || (!node.getSession().ISession.password)) {
            try {
                const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                if (values !== undefined) {
                    usrNme = values[0];
                    passWrd = values[1];
                    baseEncd = values[2];
                }
            }
            catch (error) {
                yield utils.errorHandling(error, node.getProfileName(), error.message);
            }
            if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                node.getSession().ISession.user = usrNme;
                node.getSession().ISession.password = passWrd;
                node.getSession().ISession.base64EncodedAuth = baseEncd;
                validProfile = 0;
            }
            else {
                return;
            }
            yield datasetProvider.refreshElement(node);
            yield datasetProvider.refresh();
        }
        else {
            validProfile = 0;
        }
        if (validProfile === 0) {
            try {
                let label;
                switch (node.mParent.contextValue) {
                    case (exports.FAVORITE_CONTEXT):
                        label = node.label.substring(node.label.indexOf(":") + 1).trim();
                        break;
                    case (exports.DS_PDS_CONTEXT + exports.FAV_SUFFIX):
                        label = node.mParent.label.substring(node.mParent.label.indexOf(":") + 1).trim() + "(" + node.label.trim() + ")";
                        break;
                    case (exports.DS_SESSION_CONTEXT):
                        label = node.label.trim();
                        break;
                    case (exports.DS_PDS_CONTEXT):
                        label = node.mParent.label.trim() + "(" + node.label.trim() + ")";
                        break;
                    default:
                        vscode.window.showErrorMessage(localize(83, null));
                        throw Error(localize(84, null));
                }
                log.debug(localize(85, null) + label);
                // if local copy exists, open that instead of pulling from mainframe
                const documentFilePath = getDocumentFilePath(label, node);
                if (!fs.existsSync(documentFilePath)) {
                    const response = yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: "Opening data set..."
                    }, function downloadDataset() {
                        return zowe.Download.dataSet(node.getSession(), label, {
                            file: documentFilePath,
                            returnEtag: true
                        });
                    });
                    node.setEtag(response.apiResponse.etag);
                }
                const document = yield vscode.workspace.openTextDocument(getDocumentFilePath(label, node));
                if (previewMember === true) {
                    yield vscode.window.showTextDocument(document);
                }
                else {
                    yield vscode.window.showTextDocument(document, { preview: false });
                }
            }
            catch (err) {
                log.error(localize(86, null) + JSON.stringify(err));
                yield utils.errorHandling(err, node.getProfileName(), err.message);
                throw (err);
            }
        }
    });
}
exports.openPS = openPS;
/**
 * Refreshes treeView
 *
 * @param {DataSetTree} datasetProvider
 */
function refreshAll(datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        log.debug(localize(87, null));
        datasetProvider.mSessionNodes.forEach((sessNode) => {
            if (sessNode.contextValue === exports.DS_SESSION_CONTEXT) {
                utils.labelHack(sessNode);
                sessNode.children = [];
                sessNode.dirty = true;
            }
        });
        datasetProvider.refresh();
        Profiles_1.Profiles.getInstance().refresh();
    });
}
exports.refreshAll = refreshAll;
/**
 * Refreshes the passed node with current mainframe data
 *
 * @param {ZoweNode} node - The node which represents the dataset
 */
function refreshPS(node) {
    return __awaiter(this, void 0, void 0, function* () {
        let label;
        try {
            switch (node.mParent.contextValue) {
                case (exports.FAVORITE_CONTEXT):
                    label = node.label.substring(node.label.indexOf(":") + 1).trim();
                    break;
                case (exports.DS_PDS_CONTEXT + exports.FAV_SUFFIX):
                    label = node.mParent.label.substring(node.mParent.label.indexOf(":") + 1).trim() + "(" + node.label.trim() + ")";
                    break;
                case (exports.DS_SESSION_CONTEXT):
                    label = node.label.trim();
                    break;
                case (exports.DS_PDS_CONTEXT):
                    label = node.mParent.label.trim() + "(" + node.label.trim() + ")";
                    break;
                default:
                    throw Error(localize(88, null));
            }
            const documentFilePath = getDocumentFilePath(label, node);
            const response = yield zowe.Download.dataSet(node.getSession(), label, {
                file: documentFilePath,
                returnEtag: true
            });
            node.setEtag(response.apiResponse.etag);
            const document = yield vscode.workspace.openTextDocument(documentFilePath);
            vscode.window.showTextDocument(document);
            // if there are unsaved changes, vscode won't automatically display the updates, so close and reopen
            if (document.isDirty) {
                yield vscode.commands.executeCommand("workbench.action.closeActiveEditor");
                vscode.window.showTextDocument(document);
            }
        }
        catch (err) {
            log.error(localize(89, null) + JSON.stringify(err));
            if (err.message.includes(localize(90, null))) {
                vscode.window.showInformationMessage(localize(91, null) + label +
                    localize(92, null));
            }
            else {
                yield utils.errorHandling(err, node.getProfileName(), err.message);
            }
        }
    });
}
exports.refreshPS = refreshPS;
/**
 * Refreshes the passed node with current mainframe data
 *
 * @param {ZoweUSSNode} node - The node which represents the file
 */
function refreshUSS(node) {
    return __awaiter(this, void 0, void 0, function* () {
        let label;
        switch (node.mParent.contextValue) {
            case (exports.USS_DIR_CONTEXT + exports.FAV_SUFFIX):
                label = node.fullPath;
                break;
            case (exports.USS_DIR_CONTEXT):
                label = node.fullPath;
                break;
            case (exports.USS_SESSION_CONTEXT):
                label = node.label;
                break;
            default:
                vscode.window.showErrorMessage(localize(93, null));
                throw Error(localize(94, null));
        }
        try {
            const ussDocumentFilePath = getUSSDocumentFilePath(node);
            const response = yield zowe.Download.ussFile(node.getSession(), node.fullPath, {
                file: ussDocumentFilePath,
                returnEtag: true
            });
            node.setEtag(response.apiResponse.etag);
            const document = yield vscode.workspace.openTextDocument(ussDocumentFilePath);
            vscode.window.showTextDocument(document);
            // if there are unsaved changes, vscode won't automatically display the updates, so close and reopen
            if (document.isDirty) {
                yield vscode.commands.executeCommand("workbench.action.closeActiveEditor");
                vscode.window.showTextDocument(document);
            }
        }
        catch (err) {
            if (err.message.includes(localize(95, null))) {
                vscode.window.showInformationMessage(localize(96, null) + label +
                    localize(97, null));
            }
            else {
                yield utils.errorHandling(err, node.mProfileName, err.message);
            }
        }
    });
}
exports.refreshUSS = refreshUSS;
function checkForAddedSuffix(filename) {
    // identify how close to the end of the string the last . is
    const dotPos = filename.length - (1 + filename.lastIndexOf("."));
    // tslint:disable-next-line: no-magic-numbers
    return ((dotPos >= 2 && dotPos <= 4) && // if the last characters are 2 to 4 long and lower case it has been added
        ((filename.substring(filename.length - dotPos) === filename.substring(filename.length - dotPos).toLowerCase())));
}
/**
 * Uploads the file to the mainframe
 *
 * @export
 * @param {vscode.TextDocument} doc - TextDocument that is being saved
 */
function saveFile(doc, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        // Check if file is a data set, instead of some other file
        log.debug(localize(98, null) + doc.fileName);
        const docPath = path.join(doc.fileName, "..");
        log.debug("requested to save data set: " + doc.fileName);
        if (docPath.toUpperCase().indexOf(exports.DS_DIR.toUpperCase()) === -1) {
            log.debug(localize(99, null) +
                localize(100, null) + path.relative(docPath, exports.DS_DIR));
            return;
        }
        const start = path.join(exports.DS_DIR + path.sep).length;
        const ending = doc.fileName.substring(start);
        const sesName = ending.substring(0, ending.indexOf(path.sep));
        // get session from session name
        let documentSession;
        let node;
        const sesNode = (yield datasetProvider.getChildren()).find((child) => child.label.trim() === sesName);
        if (sesNode) {
            log.debug(localize(101, null));
            documentSession = sesNode.getSession();
        }
        else {
            // if saving from favorites, a session might not exist for this node
            log.debug(localize(102, null));
            const zosmfProfile = (yield Profiles_1.Profiles.getInstance()).loadNamedProfile(sesName);
            documentSession = zowe.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile);
        }
        if (documentSession == null) {
            log.error(localize(103, null));
            return vscode.window.showErrorMessage(localize(104, null));
        }
        // If not a member
        const label = doc.fileName.substring(doc.fileName.lastIndexOf(path.sep) + 1, checkForAddedSuffix(doc.fileName) ? doc.fileName.lastIndexOf(".") : doc.fileName.length);
        log.debug(localize(105, null) + label);
        if (!label.includes("(")) {
            try {
                // Checks if file still exists on server
                const response = yield zowe.List.dataSet(documentSession, label);
                if (!response.apiResponse.items.length) {
                    return vscode.window.showErrorMessage(localize(106, null));
                }
            }
            catch (err) {
                yield utils.errorHandling(err, sesName, err.message);
            }
        }
        // Get specific node based on label and parent tree (session / favorites)
        let nodes;
        let isFromFavorites;
        if (!sesNode || sesNode.children.length === 0) {
            // saving from favorites
            nodes = utils.concatChildNodes(datasetProvider.mFavorites);
            isFromFavorites = true;
        }
        else {
            // saving from session
            nodes = utils.concatChildNodes([sesNode]);
            isFromFavorites = false;
        }
        node = yield nodes.find((zNode) => {
            // dataset in Favorites
            if (zNode.contextValue === exports.DS_FAV_CONTEXT) {
                return (zNode.label === `[${sesName}]: ${label}`);
                // member in Favorites
            }
            else if (zNode.contextValue === exports.DS_MEMBER_CONTEXT && isFromFavorites) {
                const zNodeDetails = getProfileAndDataSetName(zNode);
                return (`${zNodeDetails.profileName}(${zNodeDetails.dataSetName})` === `[${sesName}]: ${label}`);
            }
            else if (zNode.contextValue === exports.DS_MEMBER_CONTEXT && !isFromFavorites) {
                const zNodeDetails = getProfileAndDataSetName(zNode);
                return (`${zNodeDetails.profileName}(${zNodeDetails.dataSetName})` === `${label}`);
            }
            else if (zNode.contextValue === exports.DS_DS_CONTEXT) {
                return (zNode.label.trim() === label);
            }
            else {
                return false;
            }
        });
        // define upload options
        let uploadOptions;
        if (node) {
            uploadOptions = {
                etag: node.getEtag(),
                returnEtag: true
            };
        }
        try {
            const uploadResponse = yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: localize(107, null)
            }, () => {
                return zowe.Upload.pathToDataSet(documentSession, doc.fileName, label, uploadOptions); // TODO MISSED TESTING
            });
            if (uploadResponse.success) {
                vscode.window.showInformationMessage(uploadResponse.commandResponse);
                // set local etag with the new etag from the updated file on mainframe
                node.setEtag(uploadResponse.apiResponse[0].etag);
            }
            else if (!uploadResponse.success && uploadResponse.commandResponse.includes(localize(108, null))) {
                const downloadResponse = yield zowe.Download.dataSet(documentSession, label, {
                    file: doc.fileName,
                    returnEtag: true
                });
                // re-assign etag, so that it can be used with subsequent requests
                const downloadEtag = downloadResponse.apiResponse.etag;
                if (downloadEtag !== node.getEtag()) {
                    node.setEtag(downloadEtag);
                }
                vscode.window.showWarningMessage(localize(109, null));
                // Store document in a separate variable, to be used on merge conflict
                const oldDoc = doc;
                const oldDocText = oldDoc.getText();
                const startPosition = new vscode.Position(0, 0);
                const endPosition = new vscode.Position(oldDoc.lineCount, 0);
                const deleteRange = new vscode.Range(startPosition, endPosition);
                yield vscode.window.activeTextEditor.edit((editBuilder) => {
                    // re-write the old content in the editor view
                    editBuilder.delete(deleteRange);
                    editBuilder.insert(startPosition, oldDocText);
                });
                yield vscode.window.activeTextEditor.document.save();
            }
            else {
                vscode.window.showErrorMessage(uploadResponse.commandResponse);
            }
        }
        catch (err) {
            vscode.window.showErrorMessage(err.message);
        }
    });
}
exports.saveFile = saveFile;
/**
 * Uploads the file to the mainframe
 *
 * @export
 * @param {Session} session - Desired session
 * @param {vscode.TextDocument} doc - TextDocument that is being saved
 */
function saveUSSFile(doc, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        log.debug(localize(110, null) + doc.fileName);
        const start = path.join(exports.USS_DIR + path.sep).length;
        const ending = doc.fileName.substring(start);
        const sesName = ending.substring(0, ending.indexOf(path.sep));
        const remote = ending.substring(sesName.length).replace(/\\/g, "/");
        // get session from session name
        let documentSession;
        let binary;
        let node;
        const sesNode = (yield ussFileProvider.mSessionNodes.find((child) => child.mProfileName && child.mProfileName.trim() === sesName.trim()));
        if (sesNode) {
            documentSession = sesNode.getSession();
            binary = Object.keys(sesNode.binaryFiles).find((child) => child === remote) !== undefined;
        }
        // Get specific node based on label and parent tree (session / favorites)
        let nodes;
        if (!sesNode || sesNode.children.length === 0) {
            // saving from favorites
            nodes = utils.concatUSSChildNodes(ussFileProvider.mFavorites);
        }
        else {
            // saving from session
            nodes = utils.concatUSSChildNodes([sesNode]);
        }
        node = yield nodes.find((zNode) => {
            if (zNode.contextValue === exports.DS_FAV_TEXT_FILE_CONTEXT || zNode.contextValue === exports.DS_TEXT_FILE_CONTEXT) {
                return (zNode.fullPath.trim() === remote);
            }
            else {
                return false;
            }
        });
        // define upload options
        let etagToUpload;
        let returnEtag;
        if (node) {
            etagToUpload = node.getEtag();
            returnEtag = true;
        }
        try {
            const uploadResponse = yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: localize(111, null)
            }, () => {
                return zowe.Upload.fileToUSSFile(documentSession, doc.fileName, remote, binary, null, etagToUpload, returnEtag); // TODO MISSED TESTING
            });
            if (uploadResponse.success) {
                vscode.window.showInformationMessage(uploadResponse.commandResponse);
                // set local etag with the new etag from the updated file on mainframe
                node.setEtag(uploadResponse.apiResponse.etag);
                // this part never runs! zowe.Upload.fileToUSSFile doesn't return success: false, it just throws the error which is caught below!!!!!
            }
            else {
                vscode.window.showErrorMessage(uploadResponse.commandResponse);
            }
        }
        catch (err) {
            if (err.message.includes(localize(112, null))) {
                const downloadResponse = yield zowe.Download.ussFile(documentSession, node.fullPath, {
                    file: getUSSDocumentFilePath(node),
                    returnEtag: true
                });
                // re-assign etag, so that it can be used with subsequent requests
                const downloadEtag = downloadResponse.apiResponse.etag;
                if (downloadEtag !== etagToUpload) {
                    node.setEtag(downloadEtag);
                }
                vscode.window.showWarningMessage(localize(113, null));
                // Store document in a separate variable, to be used on merge conflict
                const oldDoc = doc;
                const oldDocText = oldDoc.getText();
                const startPosition = new vscode.Position(0, 0);
                const endPosition = new vscode.Position(oldDoc.lineCount, 0);
                const deleteRange = new vscode.Range(startPosition, endPosition);
                yield vscode.window.activeTextEditor.edit((editBuilder) => {
                    // re-write the old content in the editor view
                    editBuilder.delete(deleteRange);
                    editBuilder.insert(startPosition, oldDocText);
                });
                yield vscode.window.activeTextEditor.document.save();
            }
            else {
                log.error(localize(114, null) + JSON.stringify(err));
                yield utils.errorHandling(err, sesName, err.message);
            }
        }
    });
}
exports.saveUSSFile = saveUSSFile;
/**
 * Downloads and displays a file in a text editor view
 *
 * @param {ZoweUSSNode} node
 */
function openUSS(node, download = false, previewFile, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        if ((!node.getSession().ISession.user) || (!node.getSession().ISession.password)) {
            try {
                const values = yield Profiles_1.Profiles.getInstance().promptCredentials(node.mProfileName);
                if (values !== undefined) {
                    usrNme = values[0];
                    passWrd = values[1];
                    baseEncd = values[2];
                }
            }
            catch (error) {
                yield utils.errorHandling(error, node.mProfileName, error.message);
            }
            if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                node.getSession().ISession.user = usrNme;
                node.getSession().ISession.password = passWrd;
                node.getSession().ISession.base64EncodedAuth = baseEncd;
                validProfile = 0;
            }
            else {
                return;
            }
            yield ussFileProvider.refreshElement(node);
            yield ussFileProvider.refresh();
        }
        else {
            validProfile = 0;
        }
        if (validProfile === 0) {
            try {
                let label;
                switch (node.mParent.contextValue) {
                    case (exports.FAVORITE_CONTEXT):
                        label = node.label.substring(node.label.indexOf(":") + 1).trim();
                        break;
                    // Handle file path for files in directories and favorited directories
                    case (exports.USS_DIR_CONTEXT):
                    case (exports.USS_DIR_CONTEXT + exports.FAV_SUFFIX):
                        label = node.fullPath;
                        break;
                    case (exports.USS_SESSION_CONTEXT):
                        label = node.label;
                        break;
                    default:
                        vscode.window.showErrorMessage(localize(115, null));
                        throw Error(localize(116, null));
                }
                log.debug(localize(117, null) + label);
                // if local copy exists, open that instead of pulling from mainframe
                const documentFilePath = getUSSDocumentFilePath(node);
                if (download || !fs.existsSync(documentFilePath)) {
                    const chooseBinary = node.binary || (yield zowe.Utilities.isFileTagBinOrAscii(node.getSession(), node.fullPath));
                    const response = yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: "Opening USS file...",
                    }, function downloadUSSFile() {
                        return zowe.Download.ussFile(node.getSession(), node.fullPath, {
                            file: documentFilePath,
                            binary: chooseBinary,
                            returnEtag: true
                        });
                    });
                    node.setEtag(response.apiResponse.etag);
                }
                const document = yield vscode.workspace.openTextDocument(documentFilePath);
                if (previewFile === true) {
                    yield vscode.window.showTextDocument(document);
                }
                else {
                    yield vscode.window.showTextDocument(document, { preview: false });
                }
            }
            catch (err) {
                log.error(localize(118, null) + JSON.stringify(err));
                yield utils.errorHandling(err, node.mProfileName, err.message);
                throw (err);
            }
        }
    });
}
exports.openUSS = openUSS;
function modifyCommand(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const command = yield vscode.window.showInputBox({ prompt: localize(119, null) });
            if (command !== undefined) {
                const response = yield zowe.IssueCommand.issueSimple(job.session, `f ${job.job.jobname},${command}`);
                vscode.window.showInformationMessage(localize(120, null) + response.commandResponse);
            }
        }
        catch (error) {
            yield utils.errorHandling(error, null, error.message);
        }
    });
}
exports.modifyCommand = modifyCommand;
function stopCommand(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield zowe.IssueCommand.issueSimple(job.session, `p ${job.job.jobname}`);
            vscode.window.showInformationMessage(localize(121, null) + response.commandResponse);
        }
        catch (error) {
            yield utils.errorHandling(error, null, error.message);
        }
    });
}
exports.stopCommand = stopCommand;
function getSpoolContent(session, spool) {
    return __awaiter(this, void 0, void 0, function* () {
        const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(session);
        const spoolSess = zowe.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile);
        if ((!spoolSess.ISession.user) || (!spoolSess.ISession.password)) {
            try {
                const values = yield Profiles_1.Profiles.getInstance().promptCredentials(session);
                if (values !== undefined) {
                    usrNme = values[0];
                    passWrd = values[1];
                    baseEncd = values[2];
                }
            }
            catch (error) {
                yield utils.errorHandling(error, session, error.message);
            }
            if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                spoolSess.ISession.user = usrNme;
                spoolSess.ISession.password = passWrd;
                spoolSess.ISession.base64EncodedAuth = baseEncd;
                validProfile = 0;
            }
        }
        else {
            validProfile = 0;
        }
        if (validProfile === 0) {
            try {
                const uri = SpoolProvider_1.encodeJobFile(session, spool);
                const document = yield vscode.workspace.openTextDocument(uri);
                yield vscode.window.showTextDocument(document);
            }
            catch (error) {
                yield utils.errorHandling(error, session, error.message);
            }
        }
    });
}
exports.getSpoolContent = getSpoolContent;
function setOwner(job, jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const newOwner = yield vscode.window.showInputBox({ prompt: localize(122, null) });
        job.owner = newOwner;
        jobsProvider.refreshElement(job);
    });
}
exports.setOwner = setOwner;
function setPrefix(job, jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const newPrefix = yield vscode.window.showInputBox({ prompt: localize(123, null) });
        job.prefix = newPrefix;
        jobsProvider.refreshElement(job);
    });
}
exports.setPrefix = setPrefix;
function refreshJobsServer(node, jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let sesNamePrompt;
        if (node.contextValue.endsWith(exports.FAV_SUFFIX)) {
            sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
        }
        else {
            sesNamePrompt = node.label;
        }
        if ((!node.session.ISession.user) || (!node.session.ISession.password)) {
            try {
                const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                if (values !== undefined) {
                    usrNme = values[0];
                    passWrd = values[1];
                    baseEncd = values[2];
                }
            }
            catch (error) {
                yield utils.errorHandling(error, node.getProfileName(), error.message);
            }
            if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                node.session.ISession.user = usrNme;
                node.session.ISession.password = passWrd;
                node.session.ISession.base64EncodedAuth = baseEncd;
                node.owner = usrNme;
                validProfile = 0;
            }
        }
        else {
            validProfile = 0;
        }
        if (validProfile === 0) {
            yield jobsProvider.refreshElement(node);
        }
    });
}
exports.refreshJobsServer = refreshJobsServer;

//# sourceMappingURL=extension.js.map
